@@ -72,8 +72,8 @@ DocumentChangeTracker::DocumentChangeTracker( KTextEditor::Document* document )
     Q_ASSERT(document->url().isValid());
     Q_ASSERT(document);
     connect(document, SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)), SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)));
-    connect(document, SIGNAL(textRemoved(KTextEditor::Document*,KTextEditor::Range, QString)), SLOT(textRemoved(KTextEditor::Document*,KTextEditor::Range, QString)));
-    connect(document, SIGNAL(textChanged(KTextEditor::Document*,KTextEditor::Range,QString, KTextEditor::Range)), SLOT(textChanged(KTextEditor::Document*,KTextEditor::Range,QString, KTextEditor::Range)));
+    connect(document, SIGNAL(textRemoved(KTextEditor::Document*,KTextEditor::Range,QString)), SLOT(textRemoved(KTextEditor::Document*,KTextEditor::Range,QString)));
+    connect(document, SIGNAL(textChanged(KTextEditor::Document*,KTextEditor::Range,QString,KTextEditor::Range)), SLOT(textChanged(KTextEditor::Document*,KTextEditor::Range,QString,KTextEditor::Range)));
     connect(document, SIGNAL(destroyed(QObject*)), SLOT(documentDestroyed(QObject*)));
     connect(document, SIGNAL(documentSavedOrUploaded(KTextEditor::Document*,bool)), SLOT(documentSavedOrUploaded(KTextEditor::Document*,bool)));
     
@@ -81,7 +81,7 @@ DocumentChangeTracker::DocumentChangeTracker( KTextEditor::Document* document )
     Q_ASSERT(m_moving);
     m_changedRange = m_moving->newMovingRange(KTextEditor::Range(), KTextEditor::MovingRange::ExpandLeft | KTextEditor::MovingRange::ExpandRight);
     
-    connect(m_document, SIGNAL(aboutToInvalidateMovingInterfaceContent (KTextEditor::Document*)), this, SLOT(aboutToInvalidateMovingInterfaceContent (KTextEditor::Document*)));
+    connect(m_document, SIGNAL(aboutToInvalidateMovingInterfaceContent(KTextEditor::Document*)), this, SLOT(aboutToInvalidateMovingInterfaceContent(KTextEditor::Document*)));
     
     ModificationRevision::setEditorRevisionForFile(m_url, m_moving->revision());
     
@@ -144,7 +144,7 @@ bool DocumentChangeTracker::needUpdate() const
     return m_needUpdate;
 }
 
-bool DocumentChangeTracker::checkMergeTokens(const KTextEditor::Range& range, QString oldText, QString newText)
+bool DocumentChangeTracker::checkMergeTokens(const KTextEditor::Range& range, QString /*oldText*/, QString /*newText*/)
 {
     ///@todo Improve this so that it notices when we wrapped in/out of a line-comment
     ///@todo Improve this so that it really checks whether some merge-able tokens have been moved together
@@ -230,7 +230,7 @@ void DocumentChangeTracker::textInserted( Document* document, Range range )
     updateChangedRange(range);
 }
 
-void DocumentChangeTracker::textRemoved( Document* document, Range oldRange, QString oldText )
+void DocumentChangeTracker::textRemoved( Document* /*document*/, Range oldRange, QString oldText )
 {
     QString text = oldText;
     
