@@ -55,10 +55,6 @@
 
 using namespace KTextEditor;
 
-Q_DECLARE_METATYPE(KDevelop::IndexedString)
-Q_DECLARE_METATYPE(KDevelop::IndexedTopDUContext)
-Q_DECLARE_METATYPE(KDevelop::ReferencedTopDUContext)
-
 static QMutex minimumFeaturesMutex;
 static QHash<KDevelop::IndexedString, QList<KDevelop::TopDUContext::Features> > staticMinimumFeatures;
 
@@ -110,16 +106,6 @@ ParseJob::ParseJob( const KUrl &url )
         d(new ParseJobPrivate(url))
 {}
 
-void ParseJob::setTracker ( DocumentChangeTracker* tracker )
-{
-    d->tracker = tracker;
-}
-
-DocumentChangeTracker* ParseJob::tracker() const
-{
-    return d->tracker.data();
-}
-
 ParseJob::~ParseJob()
 {
     typedef QWeakPointer<QObject> QObjectPointer;
@@ -179,46 +165,6 @@ ReferencedTopDUContext ParseJob::duChain() const
     return d->duContext;
 }
 
-void ParseJob::addJob(Job* job)
-{
-    if (backgroundParser())
-        job->assignQueuePolicy(backgroundParser()->dependencyPolicy());
-
-    JobSequence::addJob(job);
-}
-
-BackgroundParser* ParseJob::backgroundParser() const
-{
-    return d->backgroundParser;
-}
-
-void ParseJob::setBackgroundParser(BackgroundParser* parser)
-{
-    if (parser) {
-        assignQueuePolicy(parser->dependencyPolicy());
-
-        for (int i = 0; i < jobListLength(); ++i)
-            jobAt(i)->assignQueuePolicy(parser->dependencyPolicy());
-
-    } else if (d->backgroundParser) {
-
-        removeQueuePolicy(d->backgroundParser->dependencyPolicy());
-
-        for (int i = 0; i < jobListLength(); ++i)
-            jobAt(i)->removeQueuePolicy(d->backgroundParser->dependencyPolicy());
-    }
-
-    d->backgroundParser = parser;
-}
-
-bool ParseJob::addDependency(ParseJob* dependency, ThreadWeaver::Job* actualDependee)
-{
-    if (!backgroundParser())
-        return false;
-
-    return backgroundParser()->dependencyPolicy()->addDependency(dependency, this, actualDependee);
-}
-
 bool ParseJob::abortRequested() const
 {
     QMutexLocker lock(d->abortMutex);
@@ -265,9 +211,9 @@ KDevelop::ProblemPointer ParseJob::readContents()
     QFileInfo fileInfo( localFile );
 
     QDateTime lastModified = fileInfo.lastModified();
-    
-    ForegroundLock lock;
-    
+
+    d->tracker = ICore::self()->languageController()->backgroundParser()->trackerForUrl(document());
+
     //Try using an artificial code-representation, which overrides everything else
     if(artificialCodeRepresentationExists(document())) {
         CodeRepresentation::Ptr repr = createCodeRepresentation(document());
@@ -275,24 +221,29 @@ KDevelop::ProblemPointer ParseJob::readContents()
         kDebug() << "took contents for " << document().str() << " from artificial code-representation";
         return KDevelop::ProblemPointer();
     }
-    
-    if(DocumentChangeTracker* t = d->tracker.data())
-    {
-        // The file is open in an editor
-        d->previousRevision = t->revisionAtLastReset();
 
-        t->reset(); // Reset the tracker to the current revision
-        Q_ASSERT(t->revisionAtLastReset());
-        
-        d->contents.contents = t->textAtLastReset().toUtf8();
-        d->contents.modification = KDevelop::ModificationRevision( lastModified, t->revisionAtLastReset()->revision() );
-        
-        d->revision = t->acquireRevision(d->contents.modification.revision);
-    }else{
+    bool hadTracker = false;
+    if(d->tracker)
+    {
+        ForegroundLock lock;
+        if(DocumentChangeTracker* t = d->tracker.data())
+        {
+            // The file is open in an editor
+            d->previousRevision = t->revisionAtLastReset();
+
+            t->reset(); // Reset the tracker to the current revision
+            Q_ASSERT(t->revisionAtLastReset());
+            
+            d->contents.contents = t->textAtLastReset().toUtf8();
+            d->contents.modification = KDevelop::ModificationRevision( lastModified, t->revisionAtLastReset()->revision() );
+            
+            d->revision = t->acquireRevision(d->contents.modification.revision);
+            hadTracker = true;
+        }
+    }
+    if (!hadTracker) {
         // We have to load the file from disk
-        
-        lock.unlock(); // Unlock the foreground lock before reading from disk, so the UI won't block due to I/O
-        
+
         QFile file( localFile );
         
         if ( !file.open( QIODevice::ReadOnly ) )
@@ -326,6 +277,7 @@ KDevelop::ProblemPointer ParseJob::readContents()
         file.close();
     }
 
+    // To make the parsing more robust, we add some zeroes to the end of the buffer.
     d->contents.contents.push_back((char)0);
     d->contents.contents.push_back((char)0);
     d->contents.contents.push_back((char)0);
