@@ -218,8 +218,6 @@ DocumentChangeSet::ChangeResult DocumentChangeSet::applyAllChanges() {
         }
     }
         
-    ModificationRevisionSet::clearCache();
-
     d->updateFiles();
     
     if(d->activationPolicy == Activate)
@@ -293,7 +291,9 @@ DocumentChangeSet::ChangeResult DocumentChangeSetPrivate::generateNewText(const
     //Create the actual new modified file
     QStringList textLines = repr->text().split('\n');
 
-    KMimeType::Ptr mime = KMimeType::findByUrl(file.toUrl());
+    KUrl url = file.toUrl();
+    
+    KMimeType::Ptr mime = KMimeType::findByUrl(url);
     
     for(int pos = sortedChanges.size()-1; pos >= 0; --pos) {
         DocumentChange& change(*sortedChanges[pos]);
@@ -307,8 +307,45 @@ DocumentChangeSet::ChangeResult DocumentChangeSetPrivate::generateNewText(const
 
             QString rightContext = QStringList(textLines.mid(change.m_range.end.line)).join("\n").mid(change.m_range.end.column);
 
-            if(formatter && formatPolicy == DocumentChangeSet::AutoFormatChanges)
-                change.m_newText = formatter->formatSource(change.m_newText, mime, leftContext, rightContext);
+            if(formatter && (formatPolicy == DocumentChangeSet::AutoFormatChanges || formatPolicy == DocumentChangeSet::AutoFormatChangesKeepIndentation))
+            {
+                QString oldNewText = change.m_newText;
+                change.m_newText = formatter->formatSource(change.m_newText, url, mime, leftContext, rightContext);
+                
+                if(formatPolicy == DocumentChangeSet::AutoFormatChangesKeepIndentation)
+                {
+                    // Reproduce the previous indentation
+                    QStringList oldLines = oldNewText.split("\n");
+                    QStringList newLines = change.m_newText.split("\n");
+                    
+                    if(oldLines.size() == newLines.size())
+                    {
+                        for(int line = 0; line < newLines.size(); ++line)
+                        {
+                            // Keep the previous indentation
+                            QString oldIndentation;
+                            for(int a = 0; a < oldLines[line].size(); ++a)
+                                if(oldLines[line][a].isSpace())
+                                    oldIndentation.append(oldLines[line][a]);
+                                else
+                                    break;
+
+                            int newIndentationLength = 0;
+
+                            for(int a = 0; a < newLines[line].size(); ++a)
+                                if(newLines[line][a].isSpace())
+                                    newIndentationLength = a;
+                                else
+                                    break;
+                            
+                            newLines[line].replace(0, newIndentationLength, oldIndentation);
+                        }
+                        change.m_newText = newLines.join("\n");
+                    }else{
+                        kDebug() << "Cannot keep the indentation because the line count has changed" << oldNewText;
+                    }
+                }
+            }
             
             textLines[change.m_range.start.line].replace(change.m_range.start.column, change.m_range.end.column-change.m_range.start.column, change.m_newText);
         }else{
@@ -385,7 +422,27 @@ DocumentChangeSet::ChangeResult DocumentChangeSetPrivate::removeDuplicates(const
 
 void DocumentChangeSetPrivate::updateFiles()
 {
+    ModificationRevisionSet::clearCache();
+    foreach(const IndexedString& file, changes.keys())
+        ModificationRevision::clearModificationCache(file);
+    
     if(updatePolicy != DocumentChangeSet::NoUpdate && ICore::self())
+    {
+        // The active document should be updated first, so that the user sees the results instantly
+        if(ICore::self()->documentController()->activeDocument())
+            ICore::self()->languageController()->backgroundParser()->addDocument(ICore::self()->documentController()->activeDocument()->url());
+        
+        // If there are currently open documents that now need an update, update them too
+        foreach(const IndexedString& doc, ICore::self()->languageController()->backgroundParser()->managedDocuments()) {
+            DUChainReadLocker lock(DUChain::lock());
+            TopDUContext* top = DUChainUtils::standardContextForUrl(doc.toUrl(), true);
+            if((top && top->parsingEnvironmentFile() && top->parsingEnvironmentFile()->needsUpdate()) || !top) {
+                lock.unlock();
+                ICore::self()->languageController()->backgroundParser()->addDocument(doc.toUrl());
+            }
+        }
+        
+        // Eventually update _all_ affected files
         foreach(const IndexedString &file, changes.keys())
         {
                 if(!file.toUrl().isValid()) {
@@ -394,15 +451,8 @@ void DocumentChangeSetPrivate::updateFiles()
                 }
                 
                 ICore::self()->languageController()->backgroundParser()->addDocument(file.toUrl());
-                foreach(const IndexedString& doc, ICore::self()->languageController()->backgroundParser()->managedDocuments()) {
-                    DUChainReadLocker lock(DUChain::lock());
-                    TopDUContext* top = DUChainUtils::standardContextForUrl(doc.toUrl());
-                    if((top && top->parsingEnvironmentFile() && top->parsingEnvironmentFile()->needsUpdate()) || !top) {
-                        lock.unlock();
-                        ICore::self()->languageController()->backgroundParser()->addDocument(doc.toUrl());
-                    }
-                }
         }
+    }
 }
 
 }
