@@ -51,7 +51,7 @@ QValidator::State IdentifierValidator::validate (QString & input, int &) const
         return Acceptable;
     
     DUChainReadLocker lock(DUChain::lock(), 10);
-    return m_context->findLocalDeclarations(identifier, SimpleCursor::invalid(), 0, AbstractType::Ptr(), DUContext::NoFiltering).empty() ? Acceptable : Invalid;
+    return m_context->findLocalDeclarations(identifier, CursorInRevision::invalid(), 0, AbstractType::Ptr(), DUContext::NoFiltering).empty() ? Acceptable : Invalid;
 }
 
 IndexedString fetchImplementationFileForClass(const Declaration & targetClass)
@@ -106,7 +106,7 @@ IndexedString fetchImplementationFileForClass(const Declaration & targetClass)
     {
         //Return the file that has the most uses
         QList<IndexedString> tiedFiles = sorter.values(sorter.end().key());
-        QMap<IndexedString, QList<SimpleRange> > uses = realClass->uses();
+        QMap<IndexedString, QList<RangeInRevision> > uses = realClass->uses();
         
         IndexedString mostUsesFile;
         unsigned int mostUses = 0;
