@@ -41,6 +41,7 @@
 #include "specializationstore.h"
 #include "persistentsymboltable.h"
 #include "classdeclaration.h"
+#include "parsingenvironment.h"
 
 using namespace KDevelop;
 using namespace KTextEditor;
@@ -235,7 +236,33 @@ QIcon DUChainUtils::iconForDeclaration(const Declaration* dec)
   return iconForProperties(completionProperties(dec));
 }
 
-TopDUContext* DUChainUtils::standardContextForUrl(const KUrl& url) {
+TopDUContext* DUChainUtils::contentContextFromProxyContext(TopDUContext* top)
+{
+  if(!top)
+    return 0;
+  if(top->parsingEnvironmentFile() && top->parsingEnvironmentFile()->isProxyContext()) {
+    if(!top->importedParentContexts().isEmpty())
+    {
+      DUContext* ctx = top->importedParentContexts()[0].context(0);
+      if(!ctx)
+        return 0;
+      TopDUContext* ret = ctx->topContext();
+      if(!ret)
+        return 0;
+      if(ret->url() != top->url())
+        kDebug() << "url-mismatch between content and proxy:" << top->url().toUrl() << ret->url().toUrl();
+      if(ret->url() == top->url() && !ret->parsingEnvironmentFile()->isProxyContext())
+        return ret;
+    }
+    else {
+      kDebug() << "Proxy-context imports no content-context";
+    }
+  } else
+    return top;
+  return 0;
+}
+
+TopDUContext* DUChainUtils::standardContextForUrl(const KUrl& url, bool preferProxyContext) {
   KDevelop::TopDUContext* chosen = 0;
 
   QList<KDevelop::ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(url);
@@ -245,13 +272,16 @@ TopDUContext* DUChainUtils::standardContextForUrl(const KUrl& url) {
     if(!chosen)
     {
       if (language->languageSupport())
-        chosen = language->languageSupport()->standardContext(url);
+        chosen = language->languageSupport()->standardContext(url, preferProxyContext);
     }
   }
 
   if(!chosen)
-    return DUChain::self()->chainForDocument(IndexedString(url.pathOrUrl()));
+    chosen = DUChain::self()->chainForDocument(IndexedString(url.pathOrUrl()), preferProxyContext);
 
+  if(!chosen && preferProxyContext)
+    return standardContextForUrl(url, false); // Fall back to a normal context
+  
   return chosen;
 }
 
