@@ -103,7 +103,7 @@ class PersistentSetMapItemRequest {
     return m_item.hash();
   }
   
-  size_t itemSize() const {
+  uint itemSize() const {
       return m_item.itemSize();
   }
 
@@ -160,6 +160,8 @@ void PersistentSetMap<Key, Data, Handler, Hasher>::addItem(const Key& key, const
     if(alg.indexOf(data) != -1)
       return;
     
+    QMutexLocker lock(m_repository.mutex());
+    
     PersistentSetMapItem<Key, Data, Hasher>* editableItem = m_repository.dynamicItemFromIndex(index);
     
     EmbeddedTreeAddItem<Data, Handler> add(const_cast<Data*>(editableItem->data()), editableItem->dataSize(), editableItem->centralFreeItem, data);
@@ -202,6 +204,8 @@ void PersistentSetMap<Key, Data, Handler, Hasher>::removeItem(const Key& key, co
     if(alg.indexOf(data) == -1)
       return;
     
+    QMutexLocker lock(m_repository.mutex());
+    
     PersistentSetMapItem<Key, Data, Hasher>* editableItem = m_repository.dynamicItemFromIndex(index);
     
     EmbeddedTreeRemoveItem<Data, Handler> remove(const_cast<Data*>(editableItem->data()), editableItem->dataSize(), editableItem->centralFreeItem, data);
