@@ -78,7 +78,7 @@ void OutputJob::startOutput()
             view->setDelegate(m_outputId, m_outputDelegate, m_delegateOwnership);
 
             if (m_killJobOnOutputClose)
-                connect(i, SIGNAL(outputRemoved(int, int)), this, SLOT(outputViewRemoved(int, int)));
+                connect(i, SIGNAL(outputRemoved(int,int)), this, SLOT(outputViewRemoved(int,int)));
 
             if (m_verbosity == OutputJob::Verbose)
                 view->raiseOutput(m_outputId);
@@ -161,5 +161,9 @@ OutputJob::OutputJobVerbosity OutputJob::verbosity() const
     return m_verbosity;
 }
 
+void OutputJob::setVerbosity(OutputJob::OutputJobVerbosity verbosity)
+{
+    m_verbosity = verbosity;
+}
 
 #include "outputjob.moc"
