@@ -67,14 +67,14 @@ static KDevelop::ProjectBaseItem* itemForPath(const QStringList& path, KDevelop:
 void NativeAppConfigPage::loadFromConfiguration(const KConfigGroup& cfg, KDevelop::IProject* project ) 
 {
     bool b = blockSignals( true );
+    projectTarget->setBaseItem( project ? project->projectItem() : 0, true);
+    projectTarget->setCurrentItemPath( cfg.readEntry( ExecutePlugin::projectTargetEntry, QStringList() ) );
+    executablePath->setUrl( cfg.readEntry( ExecutePlugin::executableEntry, KUrl() ) );
     if( cfg.readEntry( ExecutePlugin::isExecutableEntry, false ) ) 
     {
         executableRadio->setChecked( true );
-        executablePath->setUrl( cfg.readEntry( ExecutePlugin::executableEntry, KUrl() ) );
     } else 
     {
-        projectTarget->setBaseItem( project ? project->projectItem() : 0);
-        projectTarget->setCurrentItemPath( cfg.readEntry( ExecutePlugin::projectTargetEntry, QStringList() ) );
         projectTargetRadio->setChecked( true );
     }
     arguments->setText( cfg.readEntry( ExecutePlugin::argumentsEntry, "" ) );
@@ -124,14 +124,14 @@ NativeAppConfigPage::NativeAppConfigPage( QWidget* parent )
     configureEnvironment->setSelectionWidget(environment);
 
     //connect signals to changed signal
-    connect( projectTarget, SIGNAL(currentIndexChanged(const QString&)), SIGNAL(changed()) );
+    connect( projectTarget, SIGNAL(currentIndexChanged(QString)), SIGNAL(changed()) );
     connect( projectTargetRadio, SIGNAL(toggled(bool)), SIGNAL(changed()) );
     connect( executableRadio, SIGNAL(toggled(bool)), SIGNAL(changed()) );
-    connect( executablePath->lineEdit(), SIGNAL(textEdited(const QString&)), SIGNAL(changed()) );
-    connect( executablePath, SIGNAL(urlSelected(const KUrl&)), SIGNAL(changed()) );
-    connect( arguments, SIGNAL(textEdited(const QString&)), SIGNAL(changed()) );
-    connect( workingDirectory, SIGNAL(urlSelected(const KUrl&)), SIGNAL(changed()) );
-    connect( workingDirectory->lineEdit(), SIGNAL(textEdited(const QString&)), SIGNAL(changed()) );
+    connect( executablePath->lineEdit(), SIGNAL(textEdited(QString)), SIGNAL(changed()) );
+    connect( executablePath, SIGNAL(urlSelected(KUrl)), SIGNAL(changed()) );
+    connect( arguments, SIGNAL(textEdited(QString)), SIGNAL(changed()) );
+    connect( workingDirectory, SIGNAL(urlSelected(KUrl)), SIGNAL(changed()) );
+    connect( workingDirectory->lineEdit(), SIGNAL(textEdited(QString)), SIGNAL(changed()) );
     connect( environment, SIGNAL(currentIndexChanged(int)), SIGNAL(changed()) );
     connect( addDependency, SIGNAL(clicked(bool)), SLOT(addDep()) );
     connect( addDependency, SIGNAL(clicked(bool)), SIGNAL(changed()) );
@@ -253,15 +253,8 @@ void NativeAppConfigPage::saveToConfiguration( KConfigGroup cfg, KDevelop::IProj
 {
     Q_UNUSED( project );
     cfg.writeEntry( ExecutePlugin::isExecutableEntry, executableRadio->isChecked() );
-    if( executableRadio-> isChecked() )
-    {
-        cfg.writeEntry( ExecutePlugin::executableEntry, executablePath->url() );
-        cfg.deleteEntry( ExecutePlugin::projectTargetEntry );
-    } else
-    {
-        cfg.writeEntry( ExecutePlugin::projectTargetEntry, projectTarget->currentItemPath() );
-        cfg.deleteEntry( ExecutePlugin::executableEntry );
-    }
+    cfg.writeEntry( ExecutePlugin::executableEntry, executablePath->url() );
+    cfg.writeEntry( ExecutePlugin::projectTargetEntry, projectTarget->currentItemPath() );
     cfg.writeEntry( ExecutePlugin::argumentsEntry, arguments->text() );
     cfg.writeEntry( ExecutePlugin::workingDirEntry, workingDirectory->url() );
     cfg.writeEntry( ExecutePlugin::environmentGroupEntry, environment->currentProfile() );
