@@ -30,6 +30,7 @@
 #include <QFileInfo>
 #include <QDir>
 #include <QDateTime>
+#include <QtGui/QTextEdit>
 
 #include <interfaces/icore.h>
 #include <interfaces/iprojectcontroller.h>
@@ -50,6 +51,7 @@
 #include <KMessageBox>
 #include <KStandardDirs>
 #include "gitjob.h"
+#include "gitmessagehighlighter.h"
 
 K_PLUGIN_FACTORY(KDevGitFactory, registerPlugin<GitPlugin>(); )
 K_EXPORT_PLUGIN(KDevGitFactory(KAboutData("kdevgit","kdevgit",ki18n("Git"),"0.1",ki18n("A plugin to support git version control systems"), KAboutData::License_GPL)))
@@ -164,7 +166,7 @@ GitPlugin::GitPlugin( QObject *parent, const QVariantList & )
 {
     if (KStandardDirs::findExe("git").isEmpty()) {
         m_hasError = true;
-        m_errorDescription = "git is not installed";
+        m_errorDescription = i18n("git is not installed");
         return;
     }
 
@@ -282,6 +284,7 @@ bool GitPlugin::isVersionControlled(const KUrl &path)
 VcsJob* GitPlugin::init(const KUrl &directory)
 {
     DVcsJob* job = new DVcsJob(urlDir(directory), this);
+    job->setType(VcsJob::Import);
     *job << "git" << "init";
     return job;
 }
@@ -289,6 +292,7 @@ VcsJob* GitPlugin::init(const KUrl &directory)
 VcsJob* GitPlugin::createWorkingCopy(const KDevelop::VcsLocation & source, const KUrl& dest, KDevelop::IBasicVersionControl::RecursionMode)
 {
     DVcsJob* job = new GitCloneJob(urlDir(dest), this);
+    job->setType(VcsJob::Import);
     *job << "git" << "clone" << "--progress" << "--" << source.localUrl().url() << dest;
     return job;
 }
@@ -299,6 +303,7 @@ VcsJob* GitPlugin::add(const KUrl::List& localLocations, KDevelop::IBasicVersion
         return errorsFound(i18n("Did not specify the list of files"), OutputJob::Verbose);
 
     DVcsJob* job = new GitJob(dotGitDirectory(localLocations.front()), this);
+    job->setType(VcsJob::Add);
     *job << "git" << "add" << "--" << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
     return job;
 }
@@ -309,6 +314,7 @@ KDevelop::VcsJob* GitPlugin::status(const KUrl::List& localLocations, KDevelop::
         return errorsFound(i18n("Did not specify the list of files"), OutputJob::Verbose);
 
     DVcsJob* job = new GitJob(urlDir(localLocations), this, OutputJob::Silent);
+    job->setType(VcsJob::Status);
     
     if(m_oldVersion) {
         *job << "git" << "ls-files" << "-t" << "-m" << "-c" << "-o" << "-d" << "-k" << "--directory";
@@ -323,15 +329,23 @@ KDevelop::VcsJob* GitPlugin::status(const KUrl::List& localLocations, KDevelop::
 }
 
 VcsJob* GitPlugin::diff(const KUrl& fileOrDirectory, const KDevelop::VcsRevision& srcRevision, const KDevelop::VcsRevision& dstRevision,
-                        VcsDiff::Type type, IBasicVersionControl::RecursionMode recursion)
+                        VcsDiff::Type /*type*/, IBasicVersionControl::RecursionMode recursion)
 {
     //TODO: control different types
     
-    DVcsJob* job = new GitJob(dotGitDirectory(fileOrDirectory), this);
-    *job << "git" << "diff" << "--no-prefix";
-    QString revstr = revisionInterval(srcRevision, dstRevision);
-    if(!revstr.isEmpty())
-        *job << revstr;
+    DVcsJob* job = new GitJob(dotGitDirectory(fileOrDirectory), this, KDevelop::OutputJob::Silent);
+    job->setType(VcsJob::Diff);
+    *job << "git" << "diff" << "--no-prefix" << "--no-color" << "--no-ext-diff";
+    if(srcRevision.revisionType()==VcsRevision::Special
+        && dstRevision.revisionType()==VcsRevision::Special
+        && srcRevision.specialType()==VcsRevision::Base
+        && dstRevision.specialType()==VcsRevision::Working)
+        *job << "HEAD";
+    else {
+        QString revstr = revisionInterval(srcRevision, dstRevision);
+        if(!revstr.isEmpty())
+            *job << revstr;
+    }
     
     *job << "--" << (recursion == IBasicVersionControl::Recursive ? fileOrDirectory : preventRecursion(fileOrDirectory));
     
@@ -345,6 +359,7 @@ VcsJob* GitPlugin::revert(const KUrl::List& localLocations, IBasicVersionControl
         return errorsFound(i18n("Could not revert changes"), OutputJob::Verbose);
     
     DVcsJob* job = new GitJob(dotGitDirectory(localLocations.front()), this);
+    job->setType(VcsJob::Revert);
     *job << "git" << "checkout" << "--";
     *job << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
     
@@ -363,6 +378,7 @@ VcsJob* GitPlugin::commit(const QString& message,
 
     QDir dir = dotGitDirectory(localLocations.front());
     DVcsJob* job = new DVcsJob(dir, this);
+    job->setType(VcsJob::Commit);
     KUrl::List files = (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
     addNotVersionedFiles(dir, files);
     
@@ -404,6 +420,7 @@ VcsJob* GitPlugin::remove(const KUrl::List& files)
     QStringList otherStr = getLsFiles(dir, QStringList() << "--others" << "--" << files.front().toLocalFile(), KDevelop::OutputJob::Silent);
     if(otherStr.isEmpty()) {
         DVcsJob* job = new GitJob(dir, this);
+        job->setType(VcsJob::Remove);
         *job << "git" << "rm" << "-r";
         *job << "--" << files;
         return job;
@@ -415,8 +432,9 @@ VcsJob* GitPlugin::remove(const KUrl::List& files)
 VcsJob* GitPlugin::log(const KUrl& localLocation,
                 const KDevelop::VcsRevision& src, const KDevelop::VcsRevision& dst)
 {
-    DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this);
-    *job << "git" << "log" << "--date=raw";
+    DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
+    job->setType(VcsJob::Log);
+    *job << "git" << "log" << "--date=raw" /*<< "--numstat"*/;
     QString rev = revisionInterval(dst, src);
     if(!rev.isEmpty())
         *job << rev;
@@ -429,7 +447,8 @@ VcsJob* GitPlugin::log(const KUrl& localLocation,
 VcsJob* GitPlugin::log(const KUrl& localLocation, const KDevelop::VcsRevision& rev, unsigned long int limit)
 {
     DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
-    *job << "git" << "log" << "--date=raw" << toRevisionName(rev, QString());
+    job->setType(VcsJob::Log);
+    *job << "git" << "log" << "--date=raw" /*<< "--numstat"*/ << toRevisionName(rev, QString());
     if(limit>0)
         *job << QString("-%1").arg(limit);
     
@@ -441,7 +460,8 @@ VcsJob* GitPlugin::log(const KUrl& localLocation, const KDevelop::VcsRevision& r
 KDevelop::VcsJob* GitPlugin::annotate(const KUrl &localLocation, const KDevelop::VcsRevision&)
 {
     DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
-    *job << "git" << "blame" << "--porcelain";
+    job->setType(VcsJob::Annotate);
+    *job << "git" << "blame" << "--porcelain" << "-w";
     *job << "--" << localLocation;
     connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), this, SLOT(parseGitBlameOutput(KDevelop::DVcsJob*)));
     return job;
@@ -506,33 +526,6 @@ void GitPlugin::parseGitBlameOutput(DVcsJob *job)
     job->setResults(results);
 }
 
-DVcsJob* GitPlugin::switchBranch(const QString &repository, const QString &branch)
-{
-    QDir d(repository);
-    
-    if(hasModifications(d) && KMessageBox::questionYesNo(0, i18n("There are pending changes, do you want to stash them first?"))==KMessageBox::Yes) {
-        QScopedPointer<DVcsJob> stash(gitStash(d, QStringList(), KDevelop::OutputJob::Verbose));
-        stash->exec();
-    }
-    
-    DVcsJob* job = new DVcsJob(d, this);
-    *job << "git" << "checkout" << branch;
-    return job;
-}
-
-DVcsJob* GitPlugin::branch(const QString &repository, const QString &basebranch, const QString &branch,
-                             const QStringList &args)
-{
-    DVcsJob* job = new DVcsJob(QDir(repository), this, KDevelop::OutputJob::Silent);
-    *job << "git" << "branch" << args;
-    
-    *job << "--";
-    if (!branch.isEmpty())
-        *job << branch;
-    if (!basebranch.isEmpty())
-        *job << basebranch;
-    return job;
-}
 
 DVcsJob* GitPlugin::lsFiles(const QDir &repository, const QStringList &args,
                             OutputJob::OutputJobVerbosity verbosity)
@@ -549,48 +542,93 @@ DVcsJob* GitPlugin::gitStash(const QDir& repository, const QStringList& args, Ou
     return job;
 }
 
-QString GitPlugin::curBranch(const QString &repository)
+VcsJob* GitPlugin::tag(const KUrl& repository, const QString& commitMessage, const VcsRevision& rev, const QString& tagName)
 {
-    kDebug() << "Getting branch list";
-    
-    QScopedPointer<DVcsJob> job(new DVcsJob(QDir(repository), this, OutputJob::Silent));
-    *job << "git" << "symbolic-ref" << "HEAD";
-    if (job->exec() && job->status() == KDevelop::VcsJob::JobSucceeded) {
-        QString out = job->output().trimmed();
-        
-        kDebug() << "Getting branch list" << out.right(out.size()-11);
-        return out.right(out.size()-11);
-    }
-    return QString();
+    DVcsJob* job = new DVcsJob(urlDir(repository), this);
+    *job << "git" << "tag" << "-m" << commitMessage << tagName;
+    if(rev.revisionValue().isValid())
+        *job << rev.revisionValue().toString();
+    return job;
 }
 
-QStringList GitPlugin::branches(const QString &repository)
+VcsJob* GitPlugin::switchBranch(const KUrl &repository, const QString &branch)
 {
-    QStringList branchListDirty;
-    QScopedPointer<DVcsJob> job(branch(repository));
-    kDebug() << "Getting branch list";
+    QDir d=urlDir(repository);
     
-    if (job->exec() && job->status() == KDevelop::VcsJob::JobSucceeded)
-        branchListDirty = job->output().split('\n', QString::SkipEmptyParts);
-    else
-        return QStringList();
+    if(hasModifications(d) && KMessageBox::questionYesNo(0, i18n("There are pending changes, do you want to stash them first?"))==KMessageBox::Yes) {
+        QScopedPointer<DVcsJob> stash(gitStash(d, QStringList(), KDevelop::OutputJob::Verbose));
+        stash->exec();
+    }
+    
+    DVcsJob* job = new DVcsJob(d, this);
+    *job << "git" << "checkout" << branch;
+    return job;
+}
+
+VcsJob* GitPlugin::branch(const KUrl& repository, const KDevelop::VcsRevision& rev, const QString& branchName)
+{
+    Q_ASSERT(!branchName.isEmpty());
+    
+    DVcsJob* job = new DVcsJob(urlDir(repository), this);
+    *job << "git" << "branch" << "--" << branchName;
+    
+    if(!rev.prettyValue().isEmpty())
+        *job << rev.revisionValue().toString();
+    return job;
+}
+
+VcsJob* GitPlugin::deleteBranch(const KUrl& repository, const QString& branchName)
+{
+    DVcsJob* job = new DVcsJob(urlDir(repository), this, OutputJob::Silent);
+    *job << "git" << "branch" << "-D" << branchName;
+    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitCurrentBranch(KDevelop::DVcsJob*)));
+    return job;
+}
+
+VcsJob* GitPlugin::renameBranch(const KUrl& repository, const QString& oldBranchName, const QString& newBranchName)
+{
+    DVcsJob* job = new DVcsJob(urlDir(repository), this, OutputJob::Silent);
+    *job << "git" << "branch" << "-m" << newBranchName << oldBranchName;
+    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitCurrentBranch(KDevelop::DVcsJob*)));
+    return job;
+}
+
+VcsJob* GitPlugin::currentBranch(const KUrl& repository)
+{
+    DVcsJob* job = new DVcsJob(urlDir(repository), this, OutputJob::Silent);
+    *job << "git" << "symbolic-ref" << "HEAD";
+    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitCurrentBranch(KDevelop::DVcsJob*)));
+    return job;
+}
+
+void GitPlugin::parseGitCurrentBranch(DVcsJob* job)
+{
+    QString out = job->output().trimmed();
+    job->setResults(out.right(out.size()-11));
+}
+
+VcsJob* GitPlugin::branches(const KUrl &repository)
+{
+    DVcsJob* job=new DVcsJob(urlDir(repository));
+    *job << "git" << "branch" << "-a";
+    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitBranchOutput(KDevelop::DVcsJob*)));
+    return job;
+}
+
+void GitPlugin::parseGitBranchOutput(DVcsJob* job)
+{
+    QStringList branchListDirty = job->output().split('\n', QString::SkipEmptyParts);
 
     QStringList branchList;
     foreach(QString branch, branchListDirty)
     {
-        if (branch.contains('*'))
-        {
-            branch = branch.prepend('\n').section("\n*", 1);
-        }
-        else
-        {
-            branch = branch.prepend('\n').section('\n', 1);
-        }
+        if (branch.startsWith('*'))
+            branch = branch.right(branch.size()-2);
         
-        branch = branch.trimmed();
-        branchList<<branch;
+        branchList<<branch.trimmed();
     }
-    return branchList;
+    
+    job->setResults(branchList);
 }
 
 /* Few words about how this hardcore works:
@@ -779,7 +817,7 @@ QList<DVcsEvent> GitPlugin::getAllCommits(const QString &repo)
 
 void GitPlugin::initBranchHash(const QString &repo)
 {
-    QStringList branches = GitPlugin::branches(repo);
+    QStringList branches = listBranches(KUrl(repo));
     kDebug() << "BRANCHES: " << branches;
     //Now root branch is the current branch. In future it should be the longest branch
     //other commitLists are got with git-rev-lits branch ^br1 ^ br2
@@ -850,6 +888,7 @@ void GitPlugin::parseGitLogOutput(DVcsJob * job)
     QList<QVariant> commits;
     static QRegExp commitRegex( "^commit (\\w{8})\\w{32}" );
     static QRegExp infoRegex( "^(\\w+):(.*)" );
+    static QRegExp modificationsRegex("(\\d+)\\s+(\\d+)\\s+(.+)"); //18      2       shell/workingsets/workingsetwidget.cpp
 
     QString contents = job->output();
     QTextStream s(&contents);
@@ -857,8 +896,10 @@ void GitPlugin::parseGitLogOutput(DVcsJob * job)
     VcsEvent item;
     QString message;
     bool pushCommit = false;
+    
     while (!s.atEnd()) {
         QString line = s.readLine();
+        
         if (commitRegex.exactMatch(line)) {
             if (pushCommit) {
                 item.setMessage(message.trimmed());
@@ -877,6 +918,20 @@ void GitPlugin::parseGitLogOutput(DVcsJob * job)
             } else if (cap1 == "Date") {
                 item.setDate(QDateTime::fromTime_t(infoRegex.cap(2).trimmed().split(' ')[0].toUInt()));
             }
+        } else if (modificationsRegex.exactMatch(line)) {
+            int additions = modificationsRegex.cap(1).toUInt();
+            int removals = modificationsRegex.cap(2).toUInt();
+            QString filename = modificationsRegex.cap(3);
+            
+            VcsItemEvent::Actions a;
+            if(additions>0 || removals>0)
+                a=VcsItemEvent::Modified;
+            
+            VcsItemEvent itemEvent;
+            itemEvent.setActions(a);
+            itemEvent.setRepositoryLocation(filename);
+            
+            item.addItem(itemEvent);
         } else if (line.startsWith("    ")) {
             message += line.remove(0, 4);
             message += '\n';
@@ -997,10 +1052,14 @@ void GitPlugin::parseGitStatusOutput(DVcsJob* job)
 void GitPlugin::parseGitVersionOutput(DVcsJob* job)
 {
     QStringList versionString = job->output().trimmed().split(' ').last().split('.');
-    static QList<int> minimumVersion = QList<int>() << 1 << 7;
-    
+    static const QList<int> minimumVersion = QList<int>() << 1 << 7;
     kDebug() << "checking git version" << versionString << "against" << minimumVersion;
     m_oldVersion = false;
+    if (versionString.size() < minimumVersion.size()) {
+        m_oldVersion = true;
+        kWarning() << "invalid git version string:" << job->output().trimmed();
+        return;
+    }
     foreach(int num, minimumVersion) {
         QString curr = versionString.takeFirst();
         int valcurr = curr.toInt();
@@ -1166,6 +1225,21 @@ VcsJob* GitPlugin::update(const KUrl::List& localLocations, const KDevelop::VcsR
     }
 }
 
+void GitPlugin::setupCommitMessageEditor(const KUrl& localLocation, QTextEdit* editor) const
+{
+    new GitMessageHighlighter(editor);
+    QFile mergeMsgFile(dotGitDirectory(localLocation).filePath(".git/MERGE_MSG"));
+    // Some limit on the file size should be set since whole content is going to be read into
+    // the memory. 1Mb seems to be good value since it's rather strange to have so huge commit
+    // message.
+    static const qint64 maxMergeMsgFileSize = 1024*1024;
+    if (!mergeMsgFile.exists() || mergeMsgFile.size() > maxMergeMsgFileSize)
+        return;
+    mergeMsgFile.open(QIODevice::ReadOnly);
+    QString mergeMsg = QString::fromLocal8Bit(mergeMsgFile.readAll());
+    editor->setPlainText(mergeMsg);
+}
+
 class GitVcsLocationWidget : public KDevelop::StandardVcsLocationWidget
 {
     public:
