@@ -1,9 +1,8 @@
 /**************************************************************************
 **                              diffparser.h
-**                              -----------------
+**                              ------------
 **      begin                   : Sun Aug  4 15:05:35 2002
-        copyright               : (C) 2001-2003 Otto Bruggeman <otto.bruggeman@home.nl>
-**
+**      Copyright 2002-2004 Otto Bruggeman <otto.bruggeman@home.nl>
 ***************************************************************************/
 /***************************************************************************
 **
@@ -14,8 +13,8 @@
 **
 ***************************************************************************/
 
-#ifndef _DIFF_PARSER_H
-#define _DIFF_PARSER_H
+#ifndef DIFFPARSER_H
+#define DIFFPARSER_H
 
 #include "parserbase.h"
 
