@@ -42,6 +42,7 @@
 #include <interfaces/iselectioncontroller.h>
 #include <interfaces/context.h>
 #include <interfaces/icore.h>
+#include <interfaces/isession.h>
 #include <interfaces/iprojectcontroller.h>
 #include <interfaces/iuicontroller.h>
 #include <interfaces/iruncontroller.h>
@@ -65,8 +66,9 @@ using namespace KDevelop;
 
 //BEGIN ProjectManagerFilterAction
 
-ProjectManagerFilterAction::ProjectManagerFilterAction( QObject* parent )
+ProjectManagerFilterAction::ProjectManagerFilterAction(  const QString &initialFilter, QObject* parent )
     : KAction( parent )
+    , m_intialFilter(initialFilter)
 {
     setIcon(KIcon("view-filter"));
     setText(i18n("Filter..."));
@@ -80,11 +82,21 @@ QWidget* ProjectManagerFilterAction::createWidget( QWidget* parent )
     edit->setClickMessage(i18n("Filter..."));
     edit->setClearButtonShown(true);
     connect(edit, SIGNAL(textChanged(QString)), this, SIGNAL(filterChanged(QString)));
+    if (!m_intialFilter.isEmpty()) {
+        edit->setText(m_intialFilter);
+    }
+
     return edit;
 }
 
 //END ProjectManagerFilterAction
 
+static const char* sessionConfigGroup = "ProjectManagerView";
+static const char* splitterStateConfigKey = "splitterState";
+static const char* filterConfigKey = "filter";
+static const int projectTreeViewStrechFactor = 75; // %
+static const int projectBuildSetStrechFactor = 25; // %
+
 ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidget *parent )
         : QWidget( parent ), m_ui(new Ui::ProjectManagerView), m_plugin(plugin)
 {
@@ -92,6 +104,15 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
 
     setWindowIcon( SmallIcon( "project-development" ) );
 
+    KConfigGroup pmviewConfig(ICore::self()->activeSession()->config(), sessionConfigGroup);
+    if (pmviewConfig.hasKey(splitterStateConfigKey)) {
+        QByteArray geometry = pmviewConfig.readEntry<QByteArray>(splitterStateConfigKey, QByteArray());
+        m_ui->splitter->restoreState(geometry);
+    } else {
+        m_ui->splitter->setStretchFactor(0, projectTreeViewStrechFactor);
+        m_ui->splitter->setStretchFactor(1, projectBuildSetStrechFactor);
+    }
+
     m_syncAction = plugin->actionCollection()->action("locate_document");
     Q_ASSERT(m_syncAction);
     m_syncAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
@@ -107,7 +128,7 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
     addAction(plugin->actionCollection()->action("project_install"));
     addAction(plugin->actionCollection()->action("project_clean"));
 
-    connect(m_ui->projectTreeView, SIGNAL(activateUrl(const KUrl&)), this, SLOT(openUrl(const KUrl&)));
+    connect(m_ui->projectTreeView, SIGNAL(activateUrl(KUrl)), this, SLOT(openUrl(KUrl)));
 
 //     m_filters = new KLineEdit(this);
 //     m_filters->setClearButtonShown(true);
@@ -122,16 +143,22 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
 
     m_ui->projectTreeView->setModel( m_modelFilter );
 
-    ProjectManagerFilterAction* filterAction = new ProjectManagerFilterAction(this);
+    QString filterText;
+
+    if (pmviewConfig.hasKey(filterConfigKey)) {
+        filterText = pmviewConfig.readEntry(filterConfigKey, QString());
+    }
+
+    ProjectManagerFilterAction* filterAction = new ProjectManagerFilterAction(filterText, this);
     connect(filterAction, SIGNAL(filterChanged(QString)),
             this, SLOT(filterChanged(QString)));
     addAction(filterAction);
 
-    connect( m_ui->projectTreeView->selectionModel(), SIGNAL(selectionChanged( const QItemSelection&, const QItemSelection&) ),
-             this, SLOT(selectionChanged() ) );
-    connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*) ),
+    connect( m_ui->projectTreeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
+             this, SLOT(selectionChanged()) );
+    connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)),
              SLOT(updateSyncAction()));
-    connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*) ),
+    connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)),
              SLOT(updateSyncAction()));
     connect( qobject_cast<Sublime::MainWindow*>(KDevelop::ICore::self()->uiController()->activeMainWindow()), SIGNAL(areaChanged(Sublime::Area*)),
              SLOT(updateSyncAction()));
@@ -163,6 +190,10 @@ void ProjectManagerView::updateSyncAction()
 
 ProjectManagerView::~ProjectManagerView()
 {
+    KConfigGroup pmviewConfig(ICore::self()->activeSession()->config(), sessionConfigGroup);
+    pmviewConfig.writeEntry(splitterStateConfigKey, m_ui->splitter->saveState());
+    pmviewConfig.writeEntry(filterConfigKey, m_filterString);
+    pmviewConfig.sync();
 }
 
 QList<KDevelop::ProjectBaseItem*> ProjectManagerView::selectedItems() const
@@ -186,9 +217,13 @@ void ProjectManagerView::locateCurrentDocument()
 
     KDevelop::IDocument *doc = ICore::self()->documentController()->activeDocument();
 
-    // We should _never_ get a null pointer for the document, as
-    // the action is only enabled when there is an active document.
-    Q_ASSERT(doc);
+    if (!doc) {
+        // in theory we should never get a null pointer as the action is only enabled
+        // when there is an active document.
+        // but: in practice it can happen that you close the last document and press
+        // the shortcut to locate a doc or vice versa... so just do the failsafe thing here...
+        return;
+    }
 
     QModelIndex bestMatch;
     foreach (IProject* proj, ICore::self()->projectController()->projects()) {
@@ -222,6 +257,7 @@ void ProjectManagerView::openUrl( const KUrl& url )
 
 void ProjectManagerView::filterChanged(const QString &text)
 {
+    m_filterString = text;
     m_modelFilter->setFilterString(text);
 }
 
