@@ -43,11 +43,15 @@
 #include <interfaces/context.h>
 #include <interfaces/iplugincontroller.h>
 #include <interfaces/icore.h>
+#include <interfaces/iselectioncontroller.h>
+#include <project/interfaces/iprojectfilemanager.h>
+#include <project/interfaces/ibuildsystemmanager.h>
 
 #include "projectmanagerviewplugin.h"
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchain.h>
 #include <language/util/navigationtooltip.h>
+#include <project/projectutils.h>
 
 using namespace KDevelop;
 
@@ -63,8 +67,86 @@ ProjectTreeView::ProjectTreeView( QWidget *parent )
 
     setIndentation(10);
 
-    connect( this, SIGNAL( customContextMenuRequested( QPoint ) ), this, SLOT( popupContextMenu( QPoint ) ) );
-    connect( this, SIGNAL( doubleClicked( QModelIndex ) ), this, SLOT( slotActivated( QModelIndex ) ) );
+    setDragEnabled(true);
+    setDragDropMode(QAbstractItemView::InternalMove);
+    setAutoScroll(true);
+    setAutoExpandDelay(300);
+
+    connect( this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(popupContextMenu(QPoint)) );
+    connect( this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotActivated(QModelIndex)) );
+}
+
+QList<ProjectFileItem*> fileItemsWithin(const QList<ProjectBaseItem*> items)
+{
+   QList<ProjectFileItem*> fileItems;
+    foreach(ProjectBaseItem* item, items)
+    {
+        if (ProjectFileItem *file = item->file())
+            fileItems.append(file);
+        else if (item->folder())
+            fileItems.append(fileItemsWithin(item->children()));
+    }
+    return fileItems;
+}
+
+QList<ProjectBaseItem*> topLevelItemsWithin(QList<ProjectBaseItem*> items)
+{
+    qSort(items.begin(), items.end(), ProjectBaseItem::urlLessThan);
+    KUrl lastFolder;
+    for (int i = items.size() - 1; i >= 0; --i)
+    {
+        if (lastFolder.isParentOf(items[i]->url()))
+            items.removeAt(i);
+        else if (items[i]->folder())
+            lastFolder = items[i]->url();
+    }
+    return items;
+}
+
+template<class T>
+void filterDroppedItems(QList<T*> &items, ProjectBaseItem* dest)
+{
+    for (int i = items.size() - 1; i >= 0; --i)
+    {
+        //No drag and drop from and to same location
+        if (items[i]->parent() == dest)
+            items.removeAt(i);
+        //No moving between projects (technically feasible if the projectmanager is the same though...)
+        else if (items[i]->project() != dest->project())
+            items.removeAt(i);
+    }
+}
+
+ProjectBaseItem* ProjectTreeView::itemAtPos(QPoint pos)
+{
+    QAbstractProxyModel *proxy = qobject_cast<QAbstractProxyModel*>(model());
+    QModelIndex index = proxy->mapToSource(indexAt(pos));
+    return projectModel()->itemFromIndex(index);
+}
+
+void ProjectTreeView::dropEvent(QDropEvent* event)
+{
+    ProjectItemContext* selectionCtxt =
+            static_cast<ProjectItemContext*>(KDevelop::ICore::self()->selectionController()->currentSelection());
+    ProjectBaseItem* destItem = itemAtPos(event->pos());
+    if (destItem && (dropIndicatorPosition() == AboveItem || dropIndicatorPosition() == BelowItem))
+            destItem = destItem->parent();
+    if (selectionCtxt && destItem)
+    {
+        if (ProjectFolderItem *folder = destItem->folder())
+        {
+            QList<ProjectBaseItem*> usefulItems = topLevelItemsWithin(selectionCtxt->items());
+            filterDroppedItems(usefulItems, destItem);
+            destItem->project()->projectFileManager()->moveFilesAndFolders(usefulItems, folder);
+        }
+        else if (destItem->target() && destItem->project()->buildSystemManager())
+        {
+            QList<ProjectFileItem*> usefulItems = fileItemsWithin(selectionCtxt->items());
+            filterDroppedItems(usefulItems, destItem);
+            destItem->project()->buildSystemManager()->addFilesToTarget(usefulItems, destItem->target());
+        }
+    }
+    event->accept();
 }
 
 ProjectTreeView::~ProjectTreeView()
@@ -81,7 +163,7 @@ ProjectFolderItem *ProjectTreeView::currentFolderItem() const
 
     while ( current.isValid() )
     {
-        if ( ProjectFolderItem *folderItem = dynamic_cast<ProjectFolderItem*>( projectModel()->item( current ) ) )
+        if ( ProjectFolderItem *folderItem = dynamic_cast<ProjectFolderItem*>( projectModel()->itemFromIndex( current ) ) )
             return folderItem;
 
         current = proxy->mapFromSource(projectModel()->parent( current ));
@@ -101,7 +183,7 @@ ProjectFileItem *ProjectTreeView::currentFileItem() const
 
     while ( current.isValid() )
     {
-        if ( ProjectFileItem *fileItem = dynamic_cast<ProjectFileItem*>( projectModel()->item( current ) ) )
+        if ( ProjectFileItem *fileItem = dynamic_cast<ProjectFileItem*>( projectModel()->itemFromIndex( current ) ) )
             return fileItem;
 
         current = proxy->mapFromSource(projectModel()->parent( current ));
@@ -120,7 +202,7 @@ ProjectTargetItem *ProjectTreeView::currentTargetItem() const
 
     while ( current.isValid() )
     {
-        if ( ProjectTargetItem *targetItem = dynamic_cast<ProjectTargetItem*>( projectModel()->item( current ) ) )
+        if ( ProjectTargetItem *targetItem = dynamic_cast<ProjectTargetItem*>( projectModel()->itemFromIndex( current ) ) )
             return targetItem;
 
         current = projectModel()->parent( current );
@@ -141,14 +223,14 @@ KDevelop::ProjectModel *ProjectTreeView::projectModel() const
 void ProjectTreeView::slotActivated( const QModelIndex &index )
 {
     QAbstractProxyModel *proxy = qobject_cast<QAbstractProxyModel*>(model());
-    KDevelop::ProjectBaseItem *item = projectModel()->item( proxy->mapToSource(index) );
+    KDevelop::ProjectBaseItem *item = projectModel()->itemFromIndex( proxy->mapToSource(index) );
     if ( item && item->file() )
     {
         emit activateUrl( item->file()->url() );
     }
 }
-
-void appendActions(QMenu& menu, const QList<QAction*>& actions)
+//TODO test whether this could be replaced by projectbuildsetwidget.cpp::showContextMenu_appendActions
+void popupContextMenu_appendActions(QMenu& menu, const QList<QAction*>& actions)
 {
     menu.addActions(actions);
     menu.addSeparator();
@@ -165,7 +247,7 @@ void ProjectTreeView::popupContextMenu( const QPoint &pos )
 
         foreach( const QModelIndex& index, indexes )
         {
-            if ( KDevelop::ProjectBaseItem *item = projectModel()->item( proxy->mapToSource(index) ) )
+            if ( KDevelop::ProjectBaseItem *item = projectModel()->itemFromIndex( proxy->mapToSource(index) ) )
                 itemlist << item;
         }
     }
@@ -199,19 +281,22 @@ void ProjectTreeView::popupContextMenu( const QPoint &pos )
         runActions += ext.actions(ContextMenuExtension::RunGroup);
     }
 
-    appendActions(menu, buildActions);
-    appendActions(menu, runActions );
-    appendActions(menu, fileActions);
-    appendActions(menu, vcsActions);
-    appendActions(menu, extActions);
+    popupContextMenu_appendActions(menu, buildActions);
+    popupContextMenu_appendActions(menu, runActions );
+    popupContextMenu_appendActions(menu, fileActions);
+    popupContextMenu_appendActions(menu, vcsActions);
+    popupContextMenu_appendActions(menu, extActions);
 
-    if ( !itemlist.isEmpty() && itemlist.size() == 1 && itemlist[0]->folder() && itemlist[0]->folder()->isProjectRoot() )
+    if ( !itemlist.isEmpty() && itemlist.size() == 1 && itemlist[0]->folder() && !itemlist[0]->folder()->parent() )
     {
         KAction* projectConfig = new KAction(i18n("Open Configuration..."), this);
-        connect( projectConfig, SIGNAL( triggered() ), this, SLOT( openProjectConfig() ) );
+        connect( projectConfig, SIGNAL(triggered()), this, SLOT(openProjectConfig()) );
         projectActions << projectConfig;
     }
-    appendActions(menu, projectActions);
+    popupContextMenu_appendActions(menu, projectActions);
+    
+    if(!itemlist.isEmpty())
+        KDevelop::populateParentItemsMenu(itemlist.front(), &menu);
 
     if ( !menu.isEmpty() ) {
         menu.exec( mapToGlobal( pos ) );
@@ -237,7 +322,7 @@ bool ProjectTreeView::event(QEvent* event)
         QAbstractProxyModel *proxy = qobject_cast<QAbstractProxyModel*>(model());
         QModelIndex idx = proxy->mapToSource(idxView);
         
-        ProjectBaseItem* it=projectModel()->item(idx);
+        ProjectBaseItem* it=projectModel()->itemFromIndex(idx);
         if((m_idx!=idx || !m_tooltip) && it && it->file())
         {
             m_idx=idx;
