@@ -106,7 +106,7 @@ void AbstractFileManagerPlugin::Private::projectClosing(IProject* project)
         // see also addLotsOfFiles test
         foreach( FileManagerListJob* job, m_projectJobs[project] ) {
             kDebug(9517) << "killing project job:" << job;
-            job->kill();
+            job->abort();
         }
         m_projectJobs.remove(project);
     }
@@ -125,8 +125,8 @@ KJob* AbstractFileManagerPlugin::Private::eventuallyReadFolder( ProjectFolderIte
     q->connect( listJob, SIGNAL(finished(KJob*)),
                 q, SLOT(jobFinished(KJob*)) );
 
-    q->connect( listJob, SIGNAL(entries(FileManagerListJob*, ProjectFolderItem*, KIO::UDSEntryList, bool)),
-                q, SLOT(addJobItems(FileManagerListJob*, ProjectFolderItem*, KIO::UDSEntryList, bool)) );
+    q->connect( listJob, SIGNAL(entries(FileManagerListJob*,ProjectFolderItem*,KIO::UDSEntryList,bool)),
+                q, SLOT(addJobItems(FileManagerListJob*,ProjectFolderItem*,KIO::UDSEntryList,bool)) );
 
     return listJob;
 }
@@ -135,7 +135,7 @@ void AbstractFileManagerPlugin::Private::jobFinished(KJob* job)
 {
     FileManagerListJob* gmlJob = qobject_cast<FileManagerListJob*>(job);
     Q_ASSERT(gmlJob);
-    ifDebug(kDebug() << job << gmlJob;)
+    ifDebug(kDebug(9517) << job << gmlJob << gmlJob->item();)
     m_projectJobs[ gmlJob->item()->project() ].removeOne( gmlJob );
 }
 
@@ -246,6 +246,11 @@ void AbstractFileManagerPlugin::Private::created(const QString &path)
     KUrl parent = url.upUrl();
 
     foreach ( IProject* p, m_watchers.keys() ) {
+        if ( !p->projectItem()->model() ) {
+            // not yet finished with loading
+            // FIXME: how should this be handled? see unit test
+            continue;
+        }
         if ( !q->isValid(url, info.isDir(), p) ) {
             continue;
         }
@@ -297,6 +302,11 @@ void AbstractFileManagerPlugin::Private::deleted(const QString &path)
             ICore::self()->projectController()->closeProject(p);
             continue;
         }
+        if ( !p->projectItem()->model() ) {
+            // not yet finished with loading
+            // FIXME: how should this be handled? see unit test
+            continue;
+        }
         foreach ( ProjectFolderItem* item, p->foldersForUrl(url) ) {
             removeFolder(item);
         }
@@ -361,13 +371,27 @@ void AbstractFileManagerPlugin::Private::continueWatcher(ProjectFolderItem* fold
     m_watchers[folder->project()]->restartDirScan(folder->url().toLocalFile());
 }
 
+bool isChildItem(ProjectBaseItem* parent, ProjectBaseItem* child)
+{
+    do {
+        if (child == parent) {
+            return true;
+        }
+        child = child->parent();
+    } while(child);
+    return false;
+}
+
 void AbstractFileManagerPlugin::Private::removeFolder(ProjectFolderItem* folder)
 {
     ifDebug(kDebug(9517) << "removing folder:" << folder << folder->url();)
     foreach(FileManagerListJob* job, m_projectJobs[folder->project()]) {
-        if (job->item() == folder) {
+        if (isChildItem(folder, job->item())) {
             kDebug(9517) << "killing list job for removed folder" << job << folder->url();
-            job->kill();
+            job->abort();
+            Q_ASSERT(!m_projectJobs.value(folder->project()).contains(job));
+        } else {
+            job->removeSubDir(folder);
         }
     }
     folder->parent()->removeRow( folder->row() );
@@ -392,6 +416,7 @@ AbstractFileManagerPlugin::AbstractFileManagerPlugin( const KComponentData& inst
 
 AbstractFileManagerPlugin::~AbstractFileManagerPlugin()
 {
+    delete d;
 }
 
 IProjectFileManager::Features AbstractFileManagerPlugin::features() const
@@ -481,7 +506,7 @@ bool AbstractFileManagerPlugin::renameFile( ProjectFileItem * file, const KUrl&
     return d->rename(file, url);
 }
 
-bool AbstractFileManagerPlugin::removeFilesAndFolders(QList<ProjectBaseItem*> items)
+bool AbstractFileManagerPlugin::removeFilesAndFolders(const QList<ProjectBaseItem*> &items)
 {
     bool success = true;
     foreach(ProjectBaseItem* item, items)
@@ -509,6 +534,43 @@ bool AbstractFileManagerPlugin::removeFilesAndFolders(QList<ProjectBaseItem*> it
     return success;
 }
 
+bool AbstractFileManagerPlugin::moveFilesAndFolders(const QList< ProjectBaseItem* >& items, ProjectFolderItem* newParent)
+{
+    bool success = true;
+    foreach(ProjectBaseItem* item, items)
+    {
+        Q_ASSERT(item->folder() || item->file());
+
+        ProjectFolderItem* oldParent = getParentFolder(item);
+        d->stopWatcher(oldParent);
+        d->stopWatcher(newParent);
+
+        KUrl oldUrl = item->url();
+        KUrl newUrl = newParent->url();
+        newUrl.addPath(item->baseName());
+
+        success &= renameUrl(oldParent->project(), oldUrl, newUrl);
+        if ( success ) {
+            if (item->file()) {
+                emit fileRemoved(item->file());
+                ProjectFileItem *created = createFileItem( newParent->project(), newUrl, newParent );
+                emit fileAdded(created);
+            } else {
+                emit folderRemoved(item->folder());
+                ProjectFolderItem *created = createFolderItem( newParent->project(), newUrl, newParent );
+                emit folderAdded(created);
+            }
+            oldParent->removeRow( item->row() );
+        }
+
+        d->continueWatcher(oldParent);
+        d->continueWatcher(newParent);
+        if ( !success )
+            break;
+    }
+    return success;
+}
+
 bool AbstractFileManagerPlugin::isValid( const KUrl& /*url*/, const bool /*isFolder*/,
                                          IProject* /*project*/ ) const
 {
