@@ -89,6 +89,11 @@ QStringList joinProjectBasePath( const QStringList& partialpath, KDevelop::Proje
     return basePath + partialpath;
 }
 
+inline uint indexForUrl(const KUrl& url)
+{
+    return IndexedString::indexForString(url.pathOrUrl(KUrl::RemoveTrailingSlash));
+}
+
 class ProjectModelPrivate
 {
 public:
@@ -107,6 +112,8 @@ public:
         return model->itemFromIndex( idx );
     }
 
+    // a hash of IndexedString::indexForString(url) <-> ProjectBaseItem for fast lookup
+    QMultiHash<uint, ProjectBaseItem*> urlLookupTable;
 };
 
 class ProjectBaseItemPrivate
@@ -122,6 +129,7 @@ public:
     Qt::ItemFlags flags;
     ProjectModel* model;
     KUrl m_url;
+    QString iconName;
 };
 
 
@@ -140,7 +148,11 @@ ProjectBaseItem::ProjectBaseItem( IProject* project, const QString &name, Projec
 ProjectBaseItem::~ProjectBaseItem()
 {
     Q_D(ProjectBaseItem);
-    
+
+    if (model()) {
+        model()->d->urlLookupTable.remove(indexForUrl(d->m_url), this);
+    }
+
     if( parent() ) {
         parent()->takeRow( d->row );
     } else if( model() ) {
@@ -290,7 +302,20 @@ QString ProjectBaseItem::text() const
 void ProjectBaseItem::setModel( ProjectModel* model )
 {
     Q_D(ProjectBaseItem);
+
+    const uint urlIndex = indexForUrl(d->m_url);
+
+    if (model == d->model) {
+        return;
+    } else if (d->model) {
+        d->model->d->urlLookupTable.remove(urlIndex, this);
+    }
+
     d->model = model;
+    if (model) {
+        model->d->urlLookupTable.insert(urlIndex, this);
+    }
+
     foreach( ProjectBaseItem* item, d->children ) {
         item->setModel( model );
     }
@@ -313,6 +338,13 @@ void ProjectBaseItem::setText( const QString& text )
 
 ProjectBaseItem::RenameStatus ProjectBaseItem::rename(const QString& newname)
 {
+    if (parent()) {
+        foreach(ProjectBaseItem* sibling, parent()->children()) {
+            if (sibling->text() == newname) {
+                return ExistingItemSameName;
+            }
+        }
+    }
     setText( newname );
     return RenameOk;
 }
@@ -352,6 +384,11 @@ bool ProjectBaseItem::lessThan( const KDevelop::ProjectBaseItem* item ) const
     return false;
 }
 
+bool ProjectBaseItem::urlLessThan(ProjectBaseItem* item1, ProjectBaseItem* item2)
+{
+    return item1->url().path() < item2->url().path();
+}
+
 IProject* ProjectBaseItem::project() const
 {
     Q_D(const ProjectBaseItem);
@@ -397,9 +434,15 @@ QString ProjectBaseItem::baseName() const
 void ProjectBaseItem::setUrl( const KUrl& url )
 {
     Q_D(ProjectBaseItem);
+    const KUrl oldUrl = d->m_url;
     d->m_url = url;
     const QString baseName = url.fileName();
     setText( baseName );
+
+    if (model()) {
+        model()->d->urlLookupTable.remove(indexForUrl(oldUrl), this);
+        model()->d->urlLookupTable.insert(indexForUrl(d->m_url), this);
+    }
 }
 
 Qt::ItemFlags ProjectBaseItem::flags()
@@ -408,6 +451,11 @@ Qt::ItemFlags ProjectBaseItem::flags()
     return d->flags;
 }
 
+Qt::DropActions ProjectModel::supportedDropActions() const
+{
+    return (Qt::DropActions)(Qt::MoveAction);
+}
+
 void ProjectBaseItem::setFlags(Qt::ItemFlags flags)
 {
     Q_D(ProjectBaseItem);
@@ -508,6 +556,9 @@ void ProjectModel::clear()
 ProjectFolderItem::ProjectFolderItem( IProject* project, const KUrl & dir, ProjectBaseItem * parent )
         : ProjectBaseItem( project, dir.fileName(), parent )
 {
+    setFlags(flags() | Qt::ItemIsDropEnabled);
+    if (project && project->folder() != dir)
+        setFlags(flags() | Qt::ItemIsDragEnabled);
     setUrl( dir );
 }
 
@@ -566,7 +617,11 @@ ProjectBaseItem::RenameStatus ProjectFolderItem::rename(const QString& newname)
         //There exists a file with that name?
         if( !KIO::NetAccess::stat(dest, entry, 0) )
         {
-            if( !project() || project()->projectFileManager()->renameFolder(this, dest) )
+            if( !project() || !project()->projectFileManager() )
+            {
+                return ProjectBaseItem::rename(newname);
+            }
+            else if( project()->projectFileManager()->renameFolder(this, dest) )
             {
                 return ProjectBaseItem::RenameOk;
             }
@@ -597,6 +652,11 @@ bool ProjectFolderItem::hasFileOrFolder(const QString& name) const
     return false;
 }
 
+bool ProjectBaseItem::isProjectRoot() const
+{
+    return parent()==0;
+}
+
 ProjectBuildFolderItem::ProjectBuildFolderItem( IProject* project, const KUrl &dir, ProjectBaseItem *parent)
     : ProjectFolderItem( project, dir, parent )
 {
@@ -620,6 +680,7 @@ QString ProjectBuildFolderItem::iconName() const
 ProjectFileItem::ProjectFileItem( IProject* project, const KUrl & file, ProjectBaseItem * parent )
         : ProjectBaseItem( project, file.fileName(), parent )
 {
+    setFlags(flags() | Qt::ItemIsDragEnabled);
     setUrl( file );
     // Need to this manually here as setUrl() is virtual and hence the above
     // only calls the version in ProjectBaseItem and not ours
@@ -637,7 +698,7 @@ ProjectFileItem::~ProjectFileItem()
 
 QString ProjectFileItem::iconName() const
 {
-    return KMimeType::findByUrl(url(), 0, false, true)->iconName(url());
+    return d_ptr->iconName;
 }
 
 ProjectBaseItem::RenameStatus ProjectFileItem::rename(const QString& newname)
@@ -650,7 +711,11 @@ ProjectBaseItem::RenameStatus ProjectFileItem::rename(const QString& newname)
         //There exists a file with that name?
         if( !KIO::NetAccess::stat(dest, entry, 0) )
         {
-            if( !project() || project()->projectFileManager()->renameFile(this, dest) )
+            if( !project() || !project()->projectFileManager() )
+            {
+                return ProjectBaseItem::rename(newname);
+            }
+            else if( project()->projectFileManager()->renameFile(this, dest) )
             {
                 return ProjectBaseItem::RenameOk;
             }
@@ -673,6 +738,9 @@ QString ProjectFileItem::fileName() const
     return baseName();
 }
 
+///NOTE: this is kind of slow due to KMimeType::findByUrl
+///      maybe we should also introduce an extension-cache
+///      similar to what the language controller is doing
 void ProjectFileItem::setUrl( const KUrl& url )
 {
     if( project() ) {
@@ -682,6 +750,8 @@ void ProjectFileItem::setUrl( const KUrl& url )
     }
 
     ProjectBaseItem::setUrl( url );
+
+    d_ptr->iconName = KMimeType::findByUrl(url, 0, false, true)->iconName(url);
 }
 
 int ProjectFileItem::type() const
@@ -697,6 +767,7 @@ ProjectFileItem *ProjectFileItem::file() const
 ProjectTargetItem::ProjectTargetItem( IProject* project, const QString &name, ProjectBaseItem *parent )
     : ProjectBaseItem( project, name, parent )
 {
+    setFlags(flags() | Qt::ItemIsDropEnabled);
 }
 
 QString ProjectTargetItem::iconName() const
@@ -704,6 +775,11 @@ QString ProjectTargetItem::iconName() const
     return "system-run";
 }
 
+void ProjectTargetItem::setUrl(const KUrl& url)
+{
+    d_ptr->m_url=url;
+}
+
 int ProjectTargetItem::type() const
 {
     return ProjectBaseItem::Target;
@@ -922,6 +998,15 @@ bool ProjectModel::setData(const QModelIndex&, const QVariant&, int)
     return false;
 }
 
+QList< ProjectBaseItem* > ProjectModel::itemsForUrl ( const KUrl& url )
+{
+    if (!url.isValid()) {
+        return QList<ProjectBaseItem*>();
+    }
+
+    return d->urlLookupTable.values(indexForUrl(url));
+}
+
 void ProjectVisitor::visit ( IProject* prj )
 {
     visit( prj->projectItem() );
