@@ -118,6 +118,7 @@ class KDEVPLATFORMPROJECT_EXPORT ProjectBaseItem
         QList<ProjectFileItem*> fileList() const;
 
         virtual bool lessThan( const KDevelop::ProjectBaseItem* ) const;
+        static bool urlLessThan(KDevelop::ProjectBaseItem* item1, KDevelop::ProjectBaseItem* item2);
 
         /** @returns the @p row item in the list of children of this item or 0 if there is no such child. */
         ProjectBaseItem* child( int row ) const;
@@ -131,11 +132,6 @@ class KDEVPLATFORMPROJECT_EXPORT ProjectBaseItem
         QString text() const;
         /** @returns the row in the list of children of this items parent, or -1. */
         int row() const;
-        /**
-         * Allows to change the displayed text of this item.
-         * @param text the new text
-         */
-        void setText( const QString& text );
 
         /** @returns the number of children of this item, or 0 if there are none. */
         int rowCount() const;
@@ -179,20 +175,20 @@ class KDEVPLATFORMPROJECT_EXPORT ProjectBaseItem
         /** Get the url of this item (if any) */
         KUrl url() const;
 
-        /** Gets the basename of this url (if any) */
+        /** Gets the basename of this url (if any)
+         *  convenience function, returns the same as @c text() for most items
+         */
         QString baseName() const;
 
-        /** Gets the indexed version of the basename of this url (if any)
-            Use this for comparison as it is much faster */
-        IndexedString indexedBaseName() const;
-
         /**
          * Renames the item to the new name.
          * @returns status information wether the renaming succeeded.
          */
         virtual RenameStatus rename( const QString& newname );
         
-        /**
+        bool isProjectRoot() const;
+        
+         /**
          * Default flags: Qt::ItemIsEnabled | Qt::ItemIsSelectable
          * 
          * @returns the flags supported by the item
@@ -205,6 +201,15 @@ class KDEVPLATFORMPROJECT_EXPORT ProjectBaseItem
         void setFlags(Qt::ItemFlags flags);
 
     protected:
+        /**
+         * Allows to change the displayed text of this item.
+         *
+         * Most items assume text == baseName so this is *not* public.
+         *
+         * @param text the new text
+         */
+        void setText( const QString& text );
+
         class ProjectBaseItemPrivate* const d_ptr;
         ProjectBaseItem( ProjectBaseItemPrivate& dd );
         void setRow( int row );
@@ -273,6 +278,7 @@ public:
 
     virtual ProjectTargetItem *target() const;
     virtual QString iconName() const;
+    virtual void setUrl(const KUrl& );
 };
 
 /**
@@ -365,7 +371,12 @@ public:
     virtual bool insertRows(int row, int count, const QModelIndex& parent = QModelIndex());
     
     virtual Qt::ItemFlags flags(const QModelIndex& index) const;
+    virtual Qt::DropActions supportedDropActions() const;
 
+    /**
+     * Returns all items for the given URL.
+     */
+    QList<ProjectBaseItem*> itemsForUrl(const KUrl& url);
 private:
     class ProjectModelPrivate* const d;
     friend class ProjectBaseItem;
