@@ -88,12 +88,11 @@ private:
 void ProjectModelTest::initTestCase()
 {
     KDevelop::AutoTestShell::init();
-    KDevelop::TestCore* core = new KDevelop::TestCore();
-    core->initialize(KDevelop::Core::NoUi);
+    KDevelop::TestCore::initialize(KDevelop::Core::NoUi);
 
     qRegisterMetaType<QModelIndex>("QModelIndex");
     model = new ProjectModel( this );
-    new ModelTest( model, this );
+    modelTest = new ModelTest( model, this );
 }
 
 void ProjectModelTest::init()
@@ -104,6 +103,7 @@ void ProjectModelTest::init()
 void ProjectModelTest::cleanupTestCase()
 {
     delete model;
+    KDevelop::TestCore::shutdown();
 }
 
 void ProjectModelTest::testCreateFileSystemItems()
@@ -334,8 +334,8 @@ void ProjectModelTest::testItemSanity()
     // Check that model is properly emitting data-changes
     model->appendRow( parent );
     QCOMPARE( parent->index(), model->index(0, 0, QModelIndex()) );
-    QSignalSpy s( model, SIGNAL(dataChanged( const QModelIndex&, const QModelIndex& )) );
-    parent->setText( "newtest" );
+    QSignalSpy s( model, SIGNAL(dataChanged(QModelIndex,QModelIndex)) );
+    parent->setUrl( KUrl("file:///newtest") );
     QCOMPARE( s.count(), 1 );
     QCOMPARE( model->data( parent->index() ).toString(), QString("newtest") );
 
@@ -369,25 +369,34 @@ void ProjectModelTest::testRename()
     QFETCH( QString, expectedItemText );
     QFETCH( int, expectedRenameCode );
 
+    DummyProject* proj = new DummyProject( "DummyProject", 0 );
+    ProjectFolderItem* rootItem = new ProjectFolderItem( proj, KUrl("file:///dummyprojectfolder"), 0);
+    proj->setProjectItem( rootItem );
+    model->appendRow( rootItem );
+
+    new ProjectFileItem(proj, KUrl("existing"), rootItem);
+
     ProjectBaseItem* item = 0;
     if( itemType == ProjectBaseItem::Target ) {
-        item = new ProjectTargetItem( 0, itemText );
+        item = new ProjectTargetItem( proj, itemText, rootItem );
     } else if( itemType == ProjectBaseItem::File ) {
-        item = new ProjectFileItem( 0, KUrl(itemText) );
+        item = new ProjectFileItem( proj, KUrl(itemText), rootItem );
     } else if( itemType == ProjectBaseItem::Folder ) {
-        item = new ProjectFolderItem( 0, KUrl(itemText) );
+        item = new ProjectFolderItem( proj, KUrl(itemText), rootItem );
     } else if( itemType == ProjectBaseItem::BuildFolder ) {
-        item = new ProjectBuildFolderItem( 0, KUrl(itemText) );
+        item = new ProjectBuildFolderItem( proj, KUrl(itemText), rootItem );
     }
     Q_ASSERT( item );
-    
-    model->appendRow( item );
-    QSignalSpy s( model, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)) );
+
+    QCOMPARE(item->model(), model);
+    QSignalSpy s( model, SIGNAL(dataChanged(QModelIndex,QModelIndex)) );
     ProjectBaseItem::RenameStatus stat = item->rename( newName );
     QCOMPARE( (int)stat, expectedRenameCode );
     if( datachangesignal ) {
         QCOMPARE( s.count(), 1 );
         QCOMPARE( qvariant_cast<QModelIndex>( s.takeFirst().at(0) ), item->index() );
+    } else {
+        QCOMPARE( s.count(), 0 );
     }
     QCOMPARE( item->text(), expectedItemText );
 }
@@ -436,7 +445,7 @@ void ProjectModelTest::testRename_data()
     QTest::newRow("ExistingFileError")
     << (int)ProjectBaseItem::Folder
     << QString::fromLatin1("mynew")
-    << QString::fromLatin1("bin")
+    << QString::fromLatin1("existing")
     << false
     << QString::fromLatin1("mynew")
     << (int)ProjectBaseItem::ExistingItemSameName;
@@ -468,13 +477,29 @@ void ProjectModelTest::testAddItemInThread()
     model->appendRow( root );
     AddItemThread t( root );
     SignalReceiver check( model );
-    connect( model, SIGNAL(rowsInserted( const QModelIndex&, int, int )), &check, SLOT(rowsInserted(const QModelIndex&, int, int)), Qt::DirectConnection );
-    KDevelop::KDevSignalSpy spy( &t, SIGNAL( addedItems() ), Qt::QueuedConnection );
+    connect( model, SIGNAL(rowsInserted(QModelIndex,int,int)), &check, SLOT(rowsInserted(QModelIndex,int,int)), Qt::DirectConnection );
+    KDevelop::KDevSignalSpy spy( &t, SIGNAL(addedItems()), Qt::QueuedConnection );
     t.start();
     QVERIFY(spy.wait( 10000 ));
     QCOMPARE( qApp->thread(), check.threadOfSignalEmission() );
 }
 
+void ProjectModelTest::testDeleteLots()
+{
+    delete modelTest;
+
+    const int items = 10000;
+    ProjectBaseItem* root = new ProjectBaseItem( 0, "root", 0 );
+    model->appendRow( root );
+    for ( int i = 0; i < items; ++i ) {
+        new ProjectBaseItem( 0, QString::number(i), root );
+    }
+    QCOMPARE(root->children().size(), items);
+    delete root;
+
+    modelTest = new ModelTest( model, this );
+}
+
 QTEST_KDEMAIN( ProjectModelTest, GUI)
 #include "projectmodeltest.moc"
 #include "moc_projectmodeltest.cpp"
