@@ -40,7 +40,8 @@ namespace Sublime {
 }
 
 namespace KDevelop {
-
+class ProjectFileItem;
+class IProject;
 class MainWindow;
 
 /**
@@ -78,7 +79,7 @@ public:
     /**Refers to the document currently active or focused.
     @return The active document.*/
     virtual IDocument* activeDocument() const;
-
+    
     virtual void activateDocument( IDocument * document, const KTextEditor::Range& range = KTextEditor::Range::invalid() );
 
     virtual void registerDocumentForMimetype( const QString&, KDevelop::IDocumentFactory* );
@@ -87,9 +88,7 @@ public:
     /// If the \a mode is not IDocument::Silent, ask the user which documents to save.
     /// Returns false if the user cancels the save dialog.
     virtual bool saveAllDocuments(IDocument::DocumentSaveMode mode);
-    bool saveAllDocumentsForWindow(MainWindow* mw, IDocument::DocumentSaveMode mode, bool currentAreaOnly = false);
-
-    void notifyDocumentClosed(IDocument* doc);
+    bool saveAllDocumentsForWindow(KParts::MainWindow* mw, IDocument::DocumentSaveMode mode, bool currentAreaOnly = false);
 
     void initialize();
 
@@ -110,19 +109,24 @@ public:
     
     virtual bool openDocument(IDocument* doc,
                               const KTextEditor::Range& range = KTextEditor::Range::invalid(),
-                              DocumentActivationParams activationParams = 0);
-                              
+                              DocumentActivationParams activationParams = 0,
+                              IDocument* buddy = 0);
+
     virtual KTextEditor::Document* globalTextEditorInstance();
     
 public Q_SLOTS:
     /**Opens a new or existing document.
     @param url The full Url of the document to open. If it is empty, a dialog to choose the document will be opened.
     @param range The location information, if applicable.
-    @param activationParams Indicates whether to fully activate the document.*/
+    @param activationParams Indicates whether to fully activate the document.
+    @param buddy The buddy document
+    @return The opened document
+    */
     virtual Q_SCRIPTABLE IDocument* openDocument( const KUrl &url,
             const KTextEditor::Range& range = KTextEditor::Range::invalid(),
             DocumentActivationParams activationParams = 0,
-            const QString& encoding = "");
+            const QString& encoding = "",
+            IDocument* buddy = 0 );
 
     virtual Q_SCRIPTABLE IDocument* openDocumentFromText( const QString& data );
     
@@ -135,10 +139,26 @@ public Q_SLOTS:
     void closeAllOtherDocuments();
     void reloadAllDocuments();
 
+    // DBUS-compatible versions of openDocument
+    virtual Q_SCRIPTABLE bool openDocumentSimple( QString url );
+    // Opens a list of documents, with optional split-view separators, like: "file1 / [ file2 - fil3 ]" (see kdevplatform_shell_environment.sh)
+    virtual Q_SCRIPTABLE bool openDocumentsSimple( QStringList urls );
+    virtual Q_SCRIPTABLE bool openDocumentFromTextSimple( QString text );
+    
+    // Returns the currently active document
+    Q_SCRIPTABLE QString activeDocumentPath() const;
+
+    // Returns all open documents in the current area
+    Q_SCRIPTABLE QStringList activeDocumentPaths() const;
+    
 private Q_SLOTS:
     virtual void slotOpenDocument(const KUrl &url);
+    void slotProjectOpened(KDevelop::IProject* p);
+    void slotFileRenamed(const KUrl& oldname, KDevelop::ProjectFileItem* newitem);
+    void notifyDocumentClosed(Sublime::Document* doc);
 
 private:
+    bool openDocumentsWithSplitSeparators( Sublime::AreaIndex* index, QStringList urlsWithSeparators, bool& isFirstView );
     QList<IDocument*> documentsInWindow(MainWindow* mw) const;
     QList<IDocument*> documentsExclusivelyInWindow(MainWindow* mw, bool currentAreaOnly = false) const;
     QList<IDocument*> modifiedDocuments(const QList<IDocument*>& list) const;
