@@ -141,16 +141,13 @@ bool PartDocument::close(DocumentSaveMode mode)
     foreach (KParts::Part* part, d->partForView)
         part->deleteLater();
 
-    Core::self()->documentControllerInternal()->notifyDocumentClosed(this);
-
-    // Here we go...
-    deleteLater();
+    // The document will be deleted automatically if there are no views left
 
     return true;
 }
 
-bool PartDocument::closeDocument() {
-    return close();
+bool PartDocument::closeDocument(bool silent) {
+    return close(silent ? Silent : Default);
 }
 
 void PartDocument::reload()
@@ -196,9 +193,21 @@ KUrl PartDocument::url() const
 void PartDocument::setUrl(const KUrl& newUrl)
 {
     Sublime::UrlDocument::setUrl(newUrl);
+    if(!prettyName().isEmpty())
+        setTitle(prettyName());
     notifyUrlChanged();
 }
 
+void PartDocument::setPrettyName(QString name)
+{
+    KDevelop::IDocument::setPrettyName(name);
+    // Re-set the url, to trigger the whole chain
+    if(!name.isEmpty())
+        setTitle(name);
+    else
+        setTitle(url().fileName());
+}
+
 QMap<QWidget*, KParts::Part*> PartDocument::partForView() const
 {
     return d->partForView;
