@@ -60,6 +60,8 @@ public:
     virtual bool isActive() const;
     virtual DocumentState state() const;
 
+    virtual void setPrettyName(QString name);
+    
     virtual void activate(Sublime::View *activeView, KParts::MainWindow *mainWindow);
 
     virtual KTextEditor::Cursor cursorPosition() const;
@@ -67,7 +69,7 @@ public:
     virtual void setTextSelection(const KTextEditor::Range &range);
 
     //Overridden from Sublime::Document
-    virtual bool closeDocument();
+    virtual bool closeDocument(bool silent);
 protected:
     /** Gives us access to the KParts */
     QMap<QWidget*, KParts::Part*> partForView() const;
