@@ -220,13 +220,29 @@ public:
         if( !url.isValid() ) {
             return QList<ProjectBaseItem*>();
         }
-        // TODO: This is moderately efficient, but could be much faster with a
-        // QHash<QString, ProjectFolderItem> member. Would it be worth it?
-        KUrl u = topItem->url();
-        if ( u.protocol() != url.protocol() || u.host() != url.host() )
+        if ( folder.protocol() != url.protocol() || folder.host() != url.host() )
             return QList<ProjectBaseItem*>();
-    
-        return itemsForUrlInternal( url, topItem );
+
+        if (!topItem->model()) {
+            // this gets hit when the project has not yet been added to the model
+            // i.e. during import phase
+            // TODO: should we handle this somehow?
+            return QList<ProjectBaseItem*>();
+        }
+
+        Q_ASSERT(topItem->model());
+        QList<ProjectBaseItem*> items = topItem->model()->itemsForUrl(url);
+
+        QList<ProjectBaseItem*>::iterator it = items.begin();
+        while(it != items.end()) {
+            if ((*it)->project() != project) {
+                it = items.erase(it);
+            } else {
+                ++it;
+            }
+        }
+
+        return items;
     }
 
 
@@ -235,12 +251,12 @@ public:
         progress->setDone();
         ProjectController* projCtrl = Core::self()->projectControllerInternal();
         
-        loading=false;
         if(job->errorText().isEmpty()) {
+            loading=false;
             projCtrl->projectModel()->appendRow(topItem);
             projCtrl->projectImportingFinished( project );
         } else {
-            projCtrl->closeProject(project);
+            projCtrl->abortOpeningProject(project);
         }
     }
 
@@ -454,7 +470,6 @@ QString Project::projectTempFile() const
 
 KSharedConfig::Ptr Project::projectConfiguration() const
 {
-    d->m_cfg->reparseConfiguration();
     return d->m_cfg;
 }
 
@@ -515,7 +530,7 @@ bool Project::open( const KUrl& projectFileUrl_ )
     d->loadVersionControlPlugin(projectGroup);
     d->progress->setBuzzy();
     KJob* importJob = iface->createImportJob(d->topItem );
-    connect( importJob, SIGNAL( result( KJob* ) ), this, SLOT( importDone( KJob* ) ) );
+    connect( importJob, SIGNAL(result(KJob*)), this, SLOT(importDone(KJob*)) );
     Core::self()->runController()->registerJob( importJob );
     return true;
 }
