@@ -42,6 +42,7 @@ Boston, MA 02110-1301, USA.
 #include "sessiondialog.h"
 #include "shellextension.h"
 #include <interfaces/iprojectcontroller.h>
+#include <util/fileutils.h>
 #include <qapplication.h>
 #include <kprocess.h>
 #include <sublime/mainwindow.h>
@@ -59,6 +60,7 @@ Boston, MA 02110-1301, USA.
 #include <ktexteditor/document.h>
 #include <sublime/area.h>
 #include <QLabel>
+#include <QDBusConnection>
 
 
 #include <kdeversion.h>
@@ -73,36 +75,6 @@ const int recoveryStorageInterval = 10; ///@todo Make this configurable
 namespace KDevelop
 {
 
-///Recursive deletion of a directory, should go into some utility file
-static bool removeDirectory(const QDir &aDir)
-{
-  bool has_err = false;
-  if (aDir.exists())//QDir::NoDotAndDotDot
-  {
-    QFileInfoList entries = aDir.entryInfoList(QDir::NoDotAndDotDot | 
-    QDir::Dirs | QDir::Files);
-    int count = entries.size();
-    for (int idx = 0; ((idx < count) && !has_err); idx++)
-    {
-      QFileInfo entryInfo = entries[idx];
-      QString path = entryInfo.absoluteFilePath();
-      if (entryInfo.isDir())
-      {
-        has_err = !removeDirectory(QDir(path));
-      }
-      else
-      {
-        QFile file(path);
-        if (!file.remove())
-        has_err = true;
-      }
-    }
-    if (!aDir.rmdir(aDir.absolutePath()))
-      has_err = true;
-  }
-  return !has_err;
-}
-
 namespace {
     int argc = 0;
     char** argv = 0; 
@@ -159,7 +131,11 @@ public:
     }
 
     ~SessionControllerPrivate() {
-        clearRecoveryDirectory();
+        if (activeSession) {
+            // when session was active, we deleted the folder already
+            // in that case activeSession = 0
+            clearRecoveryDirectory();
+        }
     }
 
     Session* findSessionForName( const QString& name ) const
@@ -278,7 +254,7 @@ public:
         foreach( Session* s, sessionActions.keys() )
         {
             if( s->id() == QUuid( a->data().toString() ) && s != activeSession ) {
-                bool loaded = loadSessionExternally( s );
+                loadSessionExternally( s );
                 break;
             }
         }
@@ -299,7 +275,7 @@ public:
         q->actionCollection()->addAction( "session_"+s->id().toString(), a );
         q->unplugActionList( "available_sessions" );
         q->plugActionList( "available_sessions", grp->actions() );
-        connect(s, SIGNAL(nameChanged(QString, QString)), SLOT(nameChanged()));
+        connect(s, SIGNAL(nameChanged(QString,QString)), SLOT(nameChanged()));
     }
 
     SessionController* q;
@@ -325,10 +301,7 @@ public:
     
     void clearRecoveryDirectory()
     {
-        QDir recoveryDir(ownSessionDirectory() + "/recovery");
-        
-        if(recoveryDir.exists())
-            removeDirectory(recoveryDir);
+        removeDirectory(ownSessionDirectory() + "/recovery");
     }
     
 public slots:
@@ -487,20 +460,16 @@ private slots:
                 return;
         }
 
-        recoveryDir.mkpath("backup"); //Just to make sure that the recovery directory actually exists
-        {
-            //Clear the backup dir
-            QDir recoveryBackupDir(recoveryDir.path() + "/backup");
-            foreach(const QFileInfo& file, recoveryBackupDir.entryInfoList(QDir::NoDotAndDotDot | QDir::Files | QDir::Dirs))
-            {
-                QFile::remove(file.absoluteFilePath());
+        if (recoveryDir.exists("backup")) {
+            // Clear the old backup recovery directory, as we will create a new one
+            if (!removeDirectory(recoveryDir.absoluteFilePath("backup"))) {
+                kWarning() << "RECOVERY ERROR: Removing the old recovery backup directory failed in " << recoveryDir;
+                return;
             }
-            
-            bool removed = recoveryDir.rmdir("backup");
-            Q_ASSERT(removed);
         }
         
         //Make the current recovery dir the backup dir, so we always have a recovery available
+        //This may fail, because "current" might be nonexistent
         recoveryDir.rename("current", "backup");
 
         {
@@ -526,16 +495,27 @@ private slots:
                             QFile urlFile(urlFilePath);
                             urlFile.open(QIODevice::WriteOnly);
                             urlFile.write(document->url().pathOrUrl().toUtf8());
+                            urlFile.close();
                             
                             QString textFilePath = recoveryCurrentDir.path() + "/" + QString("/%1_text").arg(num);
                             QFile f(textFilePath);
                             f.open(QIODevice::WriteOnly);
                             f.write(text.toUtf8());
+                            f.close();
                             
                             currentRecoveryFiles[document->url()] =
                                         QStringList() <<  (recoveryDir.path() + "/current" + QString("/%1_url").arg(num))
                                                       << (recoveryDir.path() + "/current" + QString("/%1_text").arg(num));
                             
+                            if(urlFile.error() != QFile::NoError || f.error() != QFile::NoError)
+                            {
+                                kWarning() << "RECOVERY ERROR: Failed to write recovery for" << document->url() << "to" << textFilePath;
+                                KMessageBox::error(ICore::self()->uiController()->activeMainWindow(),
+                                                    i18n("Failed to write recovery copies to %1. Please make sure that your home directory is writable and not full. This application requires available space in the home directory to run stable. You may experience application crashes until you free up some space.", recoveryCurrentDir.path()),
+                                                    i18n("Recovery Error"));
+                                return;
+                            }
+                                                      
                             ++num;
                         }
                     }
@@ -578,28 +558,31 @@ SessionController::SessionController( QObject *parent )
     
     setXMLFile("kdevsessionui.rc");
 
+    QDBusConnection::sessionBus().registerObject( "/kdevelop/SessionController",
+        this, QDBusConnection::ExportScriptableSlots );
+
     if (Core::self()->setupFlags() & Core::NoUi) return;
 
-    KAction* action = actionCollection()->addAction( "new_session", this, SLOT( newSession() ) );
+    KAction* action = actionCollection()->addAction( "new_session", this, SLOT(newSession()) );
     action->setText( i18n("Start New Session") );
     action->setToolTip( i18n("Start a new KDevelop instance with an empty session") );
     action->setIcon(KIcon("window-new"));
 
-    action = actionCollection()->addAction( "rename_session", this, SLOT( renameSession() ) );
+    action = actionCollection()->addAction( "rename_session", this, SLOT(renameSession()) );
     action->setText( i18n("Rename Session...") );
     action->setIcon(KIcon("edit-rename"));
 
-    action = actionCollection()->addAction( "delete_session", this, SLOT( deleteSession() ) );
+    action = actionCollection()->addAction( "delete_session", this, SLOT(deleteSession()) );
     action->setText( i18n("Delete Session...") );
     action->setIcon(KIcon("edit-delete"));
 
-    action = actionCollection()->addAction( "quit", this, SIGNAL( quitSession() ) );
+    action = actionCollection()->addAction( "quit", this, SIGNAL(quitSession()) );
     action->setText( i18n("Quit") );
     action->setShortcut(Qt::CTRL | Qt::Key_Q);
     action->setIcon(KIcon("application-exit"));
 
     #if 0
-    action = actionCollection()->addAction( "configure_sessions", this, SLOT( configureSessions() ) );
+    action = actionCollection()->addAction( "configure_sessions", this, SLOT(configureSessions()) );
     action->setText( i18n("Configure Sessions...") );
     action->setToolTip( i18n("Create/Delete/Activate Sessions") );
     action->setWhatsThis( i18n( "<b>Configure Sessions</b><p>Shows a dialog to Create/Delete Sessions and set a new active session.</p>" ) );
@@ -621,7 +604,15 @@ void SessionController::startNewSession()
 
 void SessionController::cleanup()
 {
+    d->recoveryTimer.stop();
+
+    ISession* active = d->activeSession;
+    d->activeSession = 0;
+    if (active->isTemporary()) {
+        deleteSession(active->name());
+    }
     qDeleteAll(d->sessionActions);
+    d->sessionActions.clear();
 }
 
 void SessionController::initialize( const QString& session )
@@ -809,6 +800,11 @@ QString SessionController::sessionDirectory()
     return KGlobal::mainComponent().dirs()->saveLocation( "data", KGlobal::mainComponent().componentName()+"/sessions", true );
 }
 
+QString SessionController::sessionDir()
+{
+    return sessionDirectory() + "/" + activeSession()->id().toString();
+}
+
 SessionController::LockSessionState SessionController::tryLockSession(QString id)
 {
     LockSessionState ret;
@@ -881,16 +877,12 @@ void SessionChooserDialog::updateState() {
         
         if(m_model->item(row, 2))
             m_model->item(row, 2)->setText(state);
-        
-        for(int col = 0; col < 3; ++col)
-            if(m_model->item(row, col))
-                m_model->item(row, col)->setEnabled(!running);
     }
     
     m_updateStateTimer.start();
 }
 
-QString SessionController::showSessionChooserDialog(QString headerText)
+QString SessionController::showSessionChooserDialog(QString headerText, bool onlyRunning)
 {
     QTreeView* view = new QTreeView;
     QStandardItemModel* model = new QStandardItemModel(view);
@@ -920,7 +912,14 @@ QString SessionController::showSessionChooserDialog(QString headerText)
         if ( si.name.isEmpty() && si.projects.isEmpty() ) {
             continue;
         }
+
+        bool running = false;
+        if(!KDevelop::SessionController::tryLockSession(si.uuid.toString()))
+            running = true;
         
+        if(onlyRunning && !running)
+            continue;
+
         if(si.uuid.toString() == defaultSession)
             defaultRow = row;
         
@@ -928,9 +927,6 @@ QString SessionController::showSessionChooserDialog(QString headerText)
         model->setItem(row, 1, new QStandardItem(si.description));
         
         QString state;
-        bool running = false;
-        if(!KDevelop::SessionController::tryLockSession(si.uuid.toString()))
-            running = true;
         
         model->setItem(row, 2, new QStandardItem(""));
         
@@ -940,7 +936,8 @@ QString SessionController::showSessionChooserDialog(QString headerText)
         ++row;
     }
     
-    model->setItem(row, 0, new QStandardItem(i18n("Create New Session")));
+    if(!onlyRunning)
+        model->setItem(row, 0, new QStandardItem(i18n("Create New Session")));
 
     dialog.updateState();
     
@@ -974,6 +971,14 @@ QString SessionController::showSessionChooserDialog(QString headerText)
     return QString();
 }
 
+QString SessionController::sessionName()
+{
+    if(!activeSession())
+        return QString();
+    return activeSession()->description();
+}
+
+
 }
 #include "sessioncontroller.moc"
 #include "moc_sessioncontroller.cpp"
