@@ -29,6 +29,7 @@
 
 #include <KDE/KColorScheme>
 #include <KDE/KDebug>
+#include <KSqueezedTextLabel>
 
 #include <interfaces/istatus.h>
 #include <interfaces/ilanguagecontroller.h>
@@ -102,24 +103,25 @@ void StatusBar::registerStatus(QObject* status)
 {
     Q_ASSERT(qobject_cast<IStatus*>(status));
     connect(status, SIGNAL(clearMessage(KDevelop::IStatus*)), SLOT(clearMessage(KDevelop::IStatus*)));
-    connect(status, SIGNAL(showMessage(KDevelop::IStatus*, const QString&, int)), SLOT(showMessage(KDevelop::IStatus*, const QString&, int)));
+    connect(status, SIGNAL(showMessage(KDevelop::IStatus*,QString,int)), SLOT(showMessage(KDevelop::IStatus*,QString,int)));
     connect(status, SIGNAL(hideProgress(KDevelop::IStatus*)), SLOT(hideProgress(KDevelop::IStatus*)));
-    connect(status, SIGNAL(showProgress(KDevelop::IStatus*,int, int, int)), SLOT(showProgress(KDevelop::IStatus*, int, int, int)));
+    connect(status, SIGNAL(showProgress(KDevelop::IStatus*,int,int,int)), SLOT(showProgress(KDevelop::IStatus*,int,int,int)));
 
     // Don't try to connect when the status object doesn't provide an error message signal (ie. avoid warning)
-    if (status->metaObject()->indexOfSignal(SIGNAL(showErrorMessage(const QString&, int))) != -1)
-        connect(status, SIGNAL(showErrorMessage(const QString&, int)), SLOT(showErrorMessage(const QString&, int)));
+    if (status->metaObject()->indexOfSignal(SIGNAL(showErrorMessage(QString,int))) != -1)
+        connect(status, SIGNAL(showErrorMessage(QString,int)), SLOT(showErrorMessage(QString,int)));
 }
 
 QWidget* errorMessage(QWidget* parent, const QString& text)
 {
-    QLabel* label = new QLabel(parent);
+    KSqueezedTextLabel* label = new KSqueezedTextLabel(parent);
     KStatefulBrush red(KColorScheme::Window, KColorScheme::NegativeText);
     QPalette pal = label->palette();
     pal.setBrush(QPalette::WindowText, red.brush(label));
     label->setPalette(pal);
     label->setAlignment(Qt::AlignRight);
     label->setText(text);
+    label->setToolTip(text);
     return label;
 }
 
