@@ -81,13 +81,33 @@ public:
       *The contents is reconstructed, even if the area equals the currently set area. */
     void setArea(Area *area);
     
+    /**
+     * Reconstruct the view structure. This is required after significant untracked changes to the
+     * area-index structure.
+     * */
+    void reconstructViews();
+    
     /**Returns the view that is closest to the given global position, or zero.
       * */
     View* viewForPosition(QPoint globalPos) const;
     
+    /**Returns true if this main-window contains this view*/
+    bool containsView(View* view) const;
+    
+    /**Returns all areas that belong to this main-window*/
+    QList<Area*> areas() const;
+    
+    /** Sets a @p w widget that will be shown when there are no opened documents.
+     * This method takes the ownership of @p w.
+     */
+    void setBackgroundCentralWidget(QWidget* w);
+    
 public Q_SLOTS:
     /**Shows the @p view and makes it active.*/
     void activateView(Sublime::View *view);
+    /**Loads size/toolbar/menu/statusbar settings to the global configuration file.
+    Reimplement in subclasses to load more and don't forget to call inherited method.*/
+    virtual void loadSettings();
 
 Q_SIGNALS:
     /**Emitted before the area is cleared from this mainwindow.*/
@@ -113,7 +133,7 @@ protected:
 protected Q_SLOTS:
     virtual void setupAreaSelector();
     virtual void tabContextMenuRequested(Sublime::View*, KMenu*);
-    virtual void tabToolTipRequested(Sublime::View* view,QPoint point);
+    virtual void tabToolTipRequested(Sublime::View* view, Sublime::Container* container, int tab);
     /**Called whenever the user requests a context menu on a dockwidget bar.
        You can then e.g. add actions to add dockwidgets.
        Default implementation does nothing.**/
@@ -123,9 +143,6 @@ public: // FIXME?
     /**Saves size/toolbar/menu/statusbar settings to the global configuration file.
     Reimplement in subclasses to save more and don't forget to call inherited method.*/
     virtual void saveSettings();
-    /**Loads size/toolbar/menu/statusbar settings to the global configuration file.
-    Reimplement in subclasses to load more and don't forget to call inherited method.*/
-    virtual void loadSettings();
 
     /**Reimplemented to save settings.*/
     virtual bool queryClose();
@@ -135,7 +152,9 @@ public: // FIXME?
     virtual QWidget* customButtonForAreaSwitcher ( Area* area );
 
 private:
+    
     Q_PRIVATE_SLOT(d, void viewAdded(Sublime::AreaIndex*, Sublime::View*))
+    Q_PRIVATE_SLOT(d, void viewRemovedInternal(Sublime::AreaIndex*, Sublime::View*))
     Q_PRIVATE_SLOT(d, void aboutToRemoveView(Sublime::AreaIndex*, Sublime::View*))
     Q_PRIVATE_SLOT(d, void toolViewAdded(Sublime::View*, Sublime::Position))
     Q_PRIVATE_SLOT(d, void raiseToolView(Sublime::View*))
